"""vs code installer as per https://code.visualstudio.com/docs/setup/linux"""

from typing import Optional
import platform
import subprocess
from rich import box
from rich.console import Console
from rich.panel import Panel
from machineconfig.utils.schemas.installer.installer_types import InstallerData


def main(installer_data: InstallerData, version: Optional[str] = None) -> None:
    console = Console()
    _ = installer_data
    console.print(
        Panel.fit(
            "\n".join([f"🖥️  Platform: {platform.system()}", f"🔄 Version: {'latest' if version is None else version}"]),
            title="💻 VS Code Installer",
            border_style="blue",
            box=box.ROUNDED,
        )
    )

    if platform.system() == "Linux":
        console.print("🐧 Installing VS Code on Linux using official script...", style="bold")
        import machineconfig.jobs.installer as module
        from pathlib import Path

        install_script = Path(module.__file__).parent.joinpath("linux_scripts/vscode.sh").read_text(encoding="utf-8")
    elif platform.system() == "Darwin":
        console.print("🍎 Installing VS Code on macOS using Homebrew...", style="bold")
        install_script = """brew install --cask visual-studio-code"""
    elif platform.system() == "Windows":
        console.print("🪟 Installing VS Code on Windows using winget...", style="bold")
        install_script = """winget install --no-upgrade --name "Microsoft Visual Studio Code" --Id "Microsoft.VisualStudioCode" --source winget --scope user --accept-package-agreements --accept-source-agreements"""
    else:
        error_msg = f"Unsupported platform: {platform.system()}"
        console.print(
            Panel.fit(
                "\n".join([error_msg]),
                title="❌ Error",
                subtitle="⚠️ Unsupported platform",
                border_style="red",
                box=box.ROUNDED,
            )
        )
        raise NotImplementedError(error_msg)
    _ = version

    console.print("🔄 EXECUTING | Running VS Code installation...", style="bold yellow")
    try:
        subprocess.run(install_script, shell=True, text=True, check=True)
        console.print("✅ VS Code installation completed successfully", style="bold green")
    except subprocess.CalledProcessError as e:
        console.print(f"❌ Installation failed with exit code {e.returncode}", style="bold red")
        raise


if __name__ == "__main__":
    pass





from typing import Optional, Annotated
from pathlib import Path
import typer


def create_from_function(
        num_process: Annotated[int, typer.Option(..., "--num-process", "-n", help="Number of parallel processes to run")],
        path: Annotated[str, typer.Option(..., "--path", "-p", help="Path to a Python or Shell script file or a directory containing such files")] = ".",
        function: Annotated[Optional[str], typer.Option(..., "--function", "-f", help="Function to run from the Python file. If not provided, you will be prompted to choose.")] = None,
):
    from machineconfig.utils.ve import get_ve_activate_line, get_ve_path_and_ipython_profile
    from machineconfig.utils.options import choose_from_options
    from machineconfig.utils.path_helper import match_file_name, sanitize_path
    from machineconfig.utils.path_extended import PathExtended
    from machineconfig.utils.accessories import get_repo_root

    path_obj = sanitize_path(path)
    if not path_obj.exists():
        suffixes = {".py"}
        choice_file = match_file_name(sub_string=path, search_root=PathExtended.cwd(), suffixes=suffixes)
    elif path_obj.is_dir():
        from machineconfig.scripts.python.helpers_fire_command.file_wrangler import search_for_files_of_interest
        print(f"🔍 Searching recursively for Python, PowerShell and Shell scripts in directory `{path_obj}`")
        files = search_for_files_of_interest(path_obj)
        print(f"🔍 Got #{len(files)} results.")
        choice_file = choose_from_options(multi=False, options=files, fzf=True, msg="Choose one option")
        choice_file = PathExtended(choice_file)
    else:
        choice_file = path_obj


    repo_root = get_repo_root(Path(choice_file))
    print(f"💾 Selected file: {choice_file}.\nRepo root: {repo_root}")
    ve_root_from_file, ipy_profile = get_ve_path_and_ipython_profile(choice_file)
    if ipy_profile is None:
        ipy_profile = "default"
    if ve_root_from_file is None:
        raise ValueError(f"Could not determine virtual environment for file {choice_file}. Please ensure it is within a recognized project structure.")

    _activate_ve_line = get_ve_activate_line(ve_root=ve_root_from_file)

    # =========================  choosing function to run
    if function is None or function.strip() == "":
        from machineconfig.scripts.python.helpers_fire_command.fire_jobs_route_helper import choose_function_or_lines
        choice_function, choice_file, _kwargs_dict = choose_function_or_lines(choice_file, kwargs_dict={})
    else:
        choice_function = function

    from machineconfig.cluster.sessions_managers.zellij_local import run_zellij_layout
    from machineconfig.utils.schemas.layouts.layout_types import LayoutConfig
    layout: LayoutConfig = {"layoutName": "fireNprocess", "layoutTabs": []}
    for an_arg in range(num_process):
        layout["layoutTabs"].append({"tabName": f"tab{an_arg}", "startDir": str(PathExtended.cwd()), "command": f"uv run -m fire {choice_file} {choice_function} --idx={an_arg} --idx_max={num_process}"})
    print(layout)
    run_zellij_layout(layout_config=layout)




"""Utilitfrom pathlib import Path

"""

from pathlib import Path
from typing import cast, Optional, get_args, Annotated
import typer
from machineconfig.scripts.python.helpers_agents.fire_agents_helper_types import AGENTS, HOST, PROVIDER


def create(
    agent: Annotated[AGENTS, typer.Option(..., "--agents", "-a", help=f"Agent type. One of {', '.join(get_args(AGENTS)[:3])}")],
    host: Annotated[HOST, typer.Option(..., "--host", "-h", help=f"Machine to run agents on. One of {', '.join(get_args(HOST))}")],
    model: Annotated[str, typer.Option(..., "--model", "-m", help="Model to use (for crush agent).")],
    provider: Annotated[PROVIDER, typer.Option(..., "--provider", "-p", help=f"Provider to use (for crush agent). One of {', '.join(get_args(PROVIDER)[:3])}")],
    context_path: Annotated[Optional[Path], typer.Option(..., "--context-path", "-c", help="Path to the context file/folder, defaults to .ai/todo/")] = None,
    separator: Annotated[str, typer.Option(..., "--separator", "-s", help="Separator for context")] = "\n",
    agent_load: Annotated[int, typer.Option(..., "--agent-load", "-al", help="Number of tasks per prompt")] = 13,
    prompt: Annotated[Optional[str], typer.Option(..., "--prompt", "-P", help="Prompt prefix as string")] = None,
    prompt_path: Annotated[Optional[Path], typer.Option(..., "--prompt-path", "-pp", help="Path to prompt file")] = None,
    job_name: Annotated[str, typer.Option(..., "--job-name", "-j", help="Job name")] = "AI_Agents",
    separate: Annotated[bool, typer.Option(..., "--separate", "-S", help="Keep prompt material in separate file to the context.")] = True,
    output_path: Annotated[Optional[Path], typer.Option(..., "--output-path", "-o", help="Path to write the layout.json file")] = None,
    agents_dir: Annotated[Optional[Path], typer.Option(..., "--agents-dir", "-ad", help="Directory to store agent files. If not provided, will be constructed automatically.")] = None,
):

    from machineconfig.scripts.python.helpers_agents.fire_agents_help_launch import prep_agent_launch, get_agents_launch_layout
    from machineconfig.scripts.python.helpers_agents.fire_agents_load_balancer import chunk_prompts
    from machineconfig.utils.accessories import get_repo_root, randstr
    import json

    # validate mutual exclusive
    prompt_options = [prompt, prompt_path]
    provided_prompt = [opt for opt in prompt_options if opt is not None]
    if len(provided_prompt) != 1:
        raise typer.BadParameter("Exactly one of --prompt or --prompt-path must be provided")
    
    repo_root = get_repo_root(Path.cwd())
    if repo_root is None:
        typer.echo("💥 Could not determine the repository root. Please run this script from within a git repository.")
        raise typer.Exit(1)
    typer.echo(f"Operating @ {repo_root}")
    
    if context_path is None:
        context_path = repo_root / ".ai" / "todo"
        
    context_path_resolved = context_path.expanduser().resolve()
    if not context_path_resolved.exists():
        raise typer.BadParameter(f"Path does not exist: {context_path_resolved}")
    
    if context_path_resolved.is_file():
        prompt_material_re_splitted = chunk_prompts(context_path_resolved, tasks_per_prompt=agent_load, joiner=separator)
    elif context_path_resolved.is_dir():
        files = [f for f in context_path_resolved.rglob("*") if f.is_file()]
        if not files:
            raise typer.BadParameter(f"No files found in directory: {context_path_resolved}")
        concatenated = separator.join(f.read_text(encoding="utf-8") for f in files)
        prompt_material_re_splitted = [concatenated]
    else:
        raise typer.BadParameter(f"Path is neither file nor directory: {context_path_resolved}")
    
    if prompt_path is not None:
        prompt_prefix = prompt_path.read_text(encoding="utf-8")
    else:
        prompt_prefix = cast(str, prompt)
    agent_selected = agent
    if agents_dir is None: agents_dir = repo_root / ".ai" / f"tmp_prompts/{job_name}_{randstr()}"
    else:
        import shutil
        if agents_dir.exists():
            shutil.rmtree(agents_dir)
    prep_agent_launch(repo_root=repo_root, agents_dir=agents_dir, prompts_material=prompt_material_re_splitted,
                      keep_material_in_separate_file=separate,
                      prompt_prefix=prompt_prefix, machine=host, agent=agent_selected, model=model, provider=provider,
                      job_name=job_name)
    layoutfile = get_agents_launch_layout(session_root=agents_dir)    
    regenerate_py_code = f"""
#!/usr/bin/env uv run --python 3.14 --with machineconfig
agents create "{context_path_resolved}" \\
    --prompt-path "{prompt_path or ''}" \\
    --agent "{agent_selected}" \\
    --host "{host}" \\
    --job-name "{job_name}" \\
    --agent_load {agent_load} \\
    --separator "{separator}" \\
    {"--separate" if separate else ""}
"""
    (agents_dir / "aa_agents_relaunch.sh").write_text(data=regenerate_py_code, encoding="utf-8")
    layout_output_path = output_path if output_path is not None else agents_dir / "layout.json"
    layout_output_path.parent.mkdir(parents=True, exist_ok=True)
    layout_output_path.write_text(data=json.dumps(layoutfile, indent=4), encoding="utf-8")
    typer.echo(f"Created agents in {agents_dir}")
    typer.echo(f"Ceated layout in {layout_output_path}")


def collect(
    agent_dir: Annotated[str, typer.Argument(..., help="Path to the agent directory containing the prompts folder")],
    output_path: Annotated[str, typer.Argument(..., help="Path to write the concatenated material files")],
    separator: Annotated[str, typer.Option(..., help="Separator to use when concatenating material files")] = "\n",
) -> None:
    """Collect all material files from an agent directory and concatenate them."""
    if not Path(agent_dir).exists() or not Path(agent_dir).is_dir():
        raise typer.BadParameter(f"Agent directory does not exist or is not a directory: {agent_dir}")
    
    prompts_dir = Path(agent_dir) / "prompts"
    if not prompts_dir.exists():
        raise typer.BadParameter(f"Prompts directory not found: {prompts_dir}")
    
    material_files = []
    for agent_subdir in prompts_dir.iterdir():
        if agent_subdir.is_dir() and agent_subdir.name.startswith("agent_"):
            material_file = agent_subdir / f"{agent_subdir.name}_material.txt"
            if material_file.exists():
                material_files.append(material_file)
    
    if not material_files:
        typer.echo("No material files found in the agent directory.")
        return
    
    # Sort by agent index for consistent ordering
    material_files.sort(key=lambda x: int(x.parent.name.split("_")[-1]))
    
    # Read and concatenate all material files
    concatenated_content = []
    for material_file in material_files:
        content = material_file.read_text(encoding="utf-8")
        concatenated_content.append(content)
    
    result = separator.join(concatenated_content)
    
    # Write to output file
    Path(output_path).parent.mkdir(parents=True, exist_ok=True)
    Path(output_path).write_text(result, encoding="utf-8")
    typer.echo(f"Concatenated material written to {output_path}")


def template():
    from platform import system
    import machineconfig.scripts.python.helpers_agents as module
    if system() == "Linux" or system() == "Darwin":
        template_path = Path(module.__file__).parent / "templates/template.sh"
    elif system() == "Windows":
        template_path = Path(module.__file__).parent / "templates/template.ps1"
    else:
        raise typer.BadParameter(f"Unsupported OS: {system()}")

    from machineconfig.utils.accessories import get_repo_root
    repo_root = get_repo_root(Path.cwd())
    if repo_root is None:
        typer.echo("💥 Could not determine the repository root. Please run this script from within a git repository.")
        raise typer.Exit(1)
    save_path = repo_root / ".ai" / "agents" / "template_fire_agents.sh"
    save_path.parent.mkdir(parents=True, exist_ok=True)
    save_path.write_text(template_path.read_text(encoding="utf-8"), encoding="utf-8")
    typer.echo(f"Template bash script written to {save_path}")


def init_config():
    from machineconfig.scripts.python.ai.initai import add_ai_configs
    add_ai_configs(repo_root=Path.cwd())

def get_app():
    agents_app = typer.Typer(help="🤖 AI Agents management subcommands", no_args_is_help=True, add_help_option=False, add_completion=False)
    sep = "\n"
    agents_full_help = f"""
[c] Create agents layout file, ready to run.
{sep}
PROVIDER options: {', '.join(get_args(PROVIDER))}
{sep}
AGENT options: {', '.join(get_args(AGENTS))}
"""
    agents_app.command("create", no_args_is_help=True, help=agents_full_help)(create)
    agents_app.command("c", no_args_is_help=True, help="Create agents layout file, ready to run.", hidden=True)(create)
    agents_app.command("collect", no_args_is_help=True, help="[T] Collect all agent materials into a single file.")(collect)
    agents_app.command("T", no_args_is_help=True, help="Collect all agent materials into a single file.", hidden=True)(collect)
    agents_app.command("make-template", no_args_is_help=False, help="[t] Create a template for fire agents")(template)
    agents_app.command("t", no_args_is_help=False, help="Create a template for fire agents", hidden=True)(template)
    agents_app.command("make-config", no_args_is_help=False, help="[g] Initialize AI configurations in the current repository")(init_config)
    agents_app.command("g", no_args_is_help=False, help="Initialize AI configurations in the current repository", hidden=True)(init_config)
    from machineconfig.scripts.python.ai.generate_files import main
    agents_app.command("make-todo", no_args_is_help=True, help="[d] Generate a markdown file listing all Python files in the repo")(main)
    agents_app.command("d", no_args_is_help=True, help="Generate a markdown file listing all Python files in the repo", hidden=True)(main)
    from machineconfig.scripts.python.ai.generate_files import create_symlink_command
    agents_app.command(name="make-symlinks", no_args_is_help=True, help="[s] Create symlinks to the current repo in ~/code_copies/")(create_symlink_command)
    agents_app.command(name="s", no_args_is_help=True, help="Create symlinks to the current repo in ~/code_copies/", hidden=True)(create_symlink_command)
    return agents_app


def main():
    agents_app = get_app()
    # from trogon.typer import init_tui
    # agents_app_tui = init_tui(agents_app)
    agents_app()
    # agents_app_tui()


if __name__ == "__main__":  # pragma: no cover
    pass



"""
Utils
"""

import machineconfig
from pathlib import Path

EXCLUDE_DIRS = [".links", ".ai", ".venv", ".git", ".idea", ".vscode", "node_modules", "__pycache__", ".mypy_cache"]
LIBRARY_ROOT = Path(machineconfig.__file__).resolve().parent

CONFIG_ROOT = Path.home().joinpath(".config/machineconfig")
DEFAULTS_PATH = Path.home().joinpath("dotfiles/machineconfig/defaults.ini")

INSTALL_VERSION_ROOT = CONFIG_ROOT.joinpath("cli_tools_installers/versions")
INSTALL_TMP_DIR = Path.home().joinpath("tmp_results", "tmp_installers")

# LINUX_INSTALL_PATH = '/usr/local/bin'
LINUX_INSTALL_PATH = Path.home().joinpath(".local/bin").__str__()
WINDOWS_INSTALL_PATH = Path.home().joinpath("AppData/Local/Microsoft/WindowsApps").__str__()


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
import subprocess
import time
import logging
from rich.console import Console

from machineconfig.utils.schemas.layouts.layout_types import LayoutConfig


logger = logging.getLogger(__name__)
console = Console()


def restart_tab_process(tab_name: str, layout_config: LayoutConfig, session_name: str) -> bool:
    """Restart the process running in a specific Zellij tab without changing the layout.
    
    This function will:
    1. Navigate to the specified tab
    2. Send Ctrl+C to stop the running process
    3. Clear the screen
    4. Re-execute the original command
    
    The tab layout and configuration remain unchanged - only the process is restarted.
    """
    if not layout_config:
        logger.error("No layout config available. Cannot restart tab.")
        return False
    
    tab_config = None
    for tab in layout_config["layoutTabs"]:
        if tab["tabName"] == tab_name:
            tab_config = tab
            break
    
    if tab_config is None:
        logger.error(f"Tab '{tab_name}' not found in layout config.")
        console.print(f"[bold red]❌ Tab '{tab_name}' not found in layout[/bold red]")
        return False
    
    command = tab_config.get("command", "")
    if not command:
        logger.warning(f"No command configured for tab '{tab_name}'")
        console.print(f"[bold yellow]⚠️  No command to restart for tab '{tab_name}'[/bold yellow]")
        return False
    
    console.print(f"[bold cyan]🔄 Restarting tab[/bold cyan] [yellow]'{tab_name}'[/yellow]")
    console.print(f"[dim]Command: {command}[/dim]")
    
    try:
        session_arg = f"--session {session_name}" if session_name else ""
        
        subprocess.run(f"zellij {session_arg} action go-to-tab-name '{tab_name}'", shell=True, check=True, capture_output=True, text=True)
        time.sleep(0.5)
        
        subprocess.run(f"zellij {session_arg} action write-chars '\\u0003'", shell=True, check=True, capture_output=True, text=True)
        time.sleep(0.3)
        
        subprocess.run(f"zellij {session_arg} action write-chars 'clear'", shell=True, check=True, capture_output=True, text=True)
        subprocess.run(f"zellij {session_arg} action write-chars '\\n'", shell=True, check=True, capture_output=True, text=True)
        time.sleep(0.2)
        
        escaped_command = command.replace("'", "'\\''")
        subprocess.run(f"zellij {session_arg} action write-chars '{escaped_command}'", shell=True, check=True, capture_output=True, text=True)
        subprocess.run(f"zellij {session_arg} action write-chars '\\n'", shell=True, check=True, capture_output=True, text=True)
        
        console.print(f"[bold green]✅ Tab '{tab_name}' restarted successfully[/bold green]")
        return True
        
    except subprocess.CalledProcessError as e:
        logger.error(f"Failed to restart tab '{tab_name}': {e}")
        console.print(f"[bold red]❌ Failed to restart tab '{tab_name}'[/bold red]")
        console.print(f"[red]Error: {e}[/red]")
        return False
    except Exception as e:
        logger.error(f"Unexpected error while restarting tab '{tab_name}': {e}")
        console.print(f"[bold red]❌ Unexpected error: {e}[/bold red]")
        return False




"""Like yadm and dotter."""

from typing import Annotated, Literal
import typer


def main(
    file: Annotated[str, typer.Argument(help="file/folder path.")],
    method: Annotated[Literal["symlink", "copy"], typer.Option(..., "--method", "-m", help="Method to use for linking files")] = "copy",
    on_conflict: Annotated[Literal["throw-error", "overwrite-self-managed", "backup-self-managed", "overwrite-default-path", "backup-default-path"], typer.Option(..., "--on-conflict", "-o", help="Action to take on conflict")] = "throw-error",
    sensitivity: Annotated[Literal["private", "public"], typer.Option(..., "--sensitivity", "-s", help="Sensitivity of the config file.")] = "private",
    destination: Annotated[str, typer.Option("--destination", "-d", help="destination folder (override the default, use at your own risk)")] = "",) -> None:
    from rich.console import Console
    from rich.panel import Panel
    from machineconfig.utils.links import symlink_map, copy_map
    from pathlib import Path
    match sensitivity:
        case "private":
            backup_root = Path.home().joinpath("dotfiles/mapper")
        case "public":
            from machineconfig.utils.source_of_truth import CONFIG_ROOT
            backup_root = Path(CONFIG_ROOT).joinpath("dotfiles/mapper")

    console = Console()
    orig_path = Path(file).expanduser().absolute()
    if destination == "":
        new_path = backup_root.joinpath(orig_path.relative_to(Path.home()))
        new_path.parent.mkdir(parents=True, exist_ok=True)
    else:
        dest_path = Path(destination).expanduser().absolute()
        dest_path.mkdir(parents=True, exist_ok=True)
        new_path = dest_path.joinpath(orig_path.name)

    from machineconfig.utils.path_extended import PathExtended
    match method:
        case "copy":
            try:
                copy_map(config_file_default_path=PathExtended(orig_path), self_managed_config_file_path=PathExtended(new_path), on_conflict=on_conflict)
            except Exception as e:
                typer.echo(f"[red]Error:[/] {e}")
                typer.Exit(code=1)
                return
        case "symlink":
            try:
                symlink_map(config_file_default_path=PathExtended(orig_path), self_managed_config_file_path=PathExtended(new_path), on_conflict=on_conflict)
            except Exception as e:
                typer.echo(f"[red]Error:[/] {e}")
                typer.Exit(code=1)
        case _:
            raise ValueError(f"Unknown method: {method}")
    console.print(Panel("\n".join(["✅ Symbolic link created successfully!", "🔄 Add the following snippet to mapper.toml to persist this mapping:",]), title="Symlink Created", border_style="green", padding=(1, 2),))

    # mapper_snippet = "\n".join(
    #     [
    #         f"[bold]📝 Edit configuration file:[/] [cyan]nano {PathExtended(CONFIG_ROOT)}/symlinks/mapper.toml[/cyan]",
    #         "",
    #         f"[{new_path.parent.name}]",
    #         f"{orig_path.name.split('.')[0]} = {{ this = '{orig_path.collapseuser().as_posix()}', to_this = '{new_path.collapseuser().as_posix()}' }}",
    #     ]
    # )
    # console.print(
    #     Panel(
    #         mapper_snippet,
    #         title="Mapper Entry",
    #         border_style="cyan",
    #         padding=(1, 2),
    #     )
    # )


def arg_parser() -> None:
    typer.run(main)


if __name__ == "__main__":
    arg_parser()



"""Mount a remote SSHFS share on a local directory"""

from platform import system
import subprocess
from machineconfig.utils.ssh import SSH
from machineconfig.utils.path_extended import PathExtended

from machineconfig.utils.options import choose_ssh_host


def main():
    print("\n" + "=" * 50)
    print("🚀 Starting SSHFS Mounting Process")
    print("=" * 50 + "\n")

    share_info = input("🔗 Enter share path (e.g., user@host:/path) [Press Enter for interactive choice]: ")
    if share_info == "":
        print("\n🔍 Interactive mode selected for choosing share path.")
        tmp = choose_ssh_host(multi=False)
        assert isinstance(tmp, str)
        ssh = SSH(host=tmp, username=None, hostname=None, ssh_key_path=None, password=None, port=22, enable_compression=False)
        share_info = f"{ssh.username}@{ssh.hostname}:{ssh.run_shell(command='echo $HOME', verbose_output=False, description='Get home directory', strict_stderr=False, strict_return_code=True).op}/data/share_ssh"
    else:
        ssh = SSH(host=share_info.split(":")[0], username=None, hostname=None, ssh_key_path=None, password=None, port=22, enable_compression=False)

    print(f"\n🌐 Share Info: {share_info}")

    if system() == "Windows":
        print("\n🔍 Checking existing drives...")
        completed = subprocess.run(["powershell", "-Command", "net use"], capture_output=True, check=False, text=True)
        print((completed.stdout or "").strip())
        driver_letter = input(r"🖥️ Choose driver letter (e.g., Z:\\) [Avoid already used ones]: ") or "Z:\\"
    else:
        driver_letter = None

    mount_point = input(f"📂 Enter the mount point directory (e.g., /mnt/network) [Default: ~/data/mount_ssh/{ssh.hostname}]: ")
    if mount_point == "":
        mount_point = PathExtended.home().joinpath(rf"data/mount_ssh/{ssh.hostname}")

    print(f"\n📁 Mount Point: {mount_point}")

    if system() == "Linux":
        txt = """
sshfs alex@:/media/dbhdd /media/dbhdd\
"""
        print("\n🔧 Preparing SSHFS mount command for Linux...")
    elif system() == "Windows":
        txt = rf"""
net use {driver_letter} {share_info}
fusermount -u /mnt/dbhdd
"""
        print("\n🔧 Preparing SSHFS mount command for Windows...")
    else:
        raise ValueError(f"❌ Not implemented for this system: {system()}")

    # PROGRAM_PATH.write_text(txt, encoding="utf-8")
    subprocess.run(txt, shell=True, check=True)
    print("✅ Configuration saved successfully!\n")

    print("🎉 SSHFS Mounting Process Completed!\n")


if __name__ == "__main__":
    main()



# import streamlit as st
# import plotly.express as px
# import plotly.graph_objects as go
# from numpy import typing as npt
# import time
# from typing import Any

# _ = px, Any, go, npt, time, np
# st.set_page_config(layout="wide", page_title="📊 Interactive Figure Viewer", page_icon="📈")

# # Initialize session state
# if 'current_index' not in st.session_state:
#     st.session_state.current_index = 0
# if 'last_time' not in st.session_state:
#     st.session_state.last_time = time.time()

# data_path = "get_figure_placeholder.pkl"  # Placeholder for the data path

# # get_figure_placeholder
# def get_figure(data: Any) -> go.Figure:
#     return px.line(data)
# # get_figure_placeholder

# total_figures = len(data)

# def change_index(new_index: int) -> None:
#     """Update the current index with bounds checking"""
#     if 0 <= new_index < total_figures:
#         st.session_state.current_index = new_index
#         st.session_state.last_time = time.time()

# def next_figure():
#     """Navigate to the next figure"""
#     change_index(st.session_state.current_index + 1)

# def prev_figure():
#     """Navigate to the previous figure"""
#     change_index(st.session_state.current_index - 1)

# def go_to_index():
#     """Navigate to a specific index"""
#     try:
#         idx = int(st.session_state.input_index)
#         change_index(idx)
#     except ValueError:
#         st.error("""
# ❌ Invalid Input!
# Please enter a valid integer index. 🧮
#         """)

# st.title("""
# 📊 Interactive Figure Viewer
# =============================
# """)

# # Top info bar with metadata
# col_info1, col_info2, col_info3, col_info4 = st.columns([1, 1, 1, 1])
# with col_info1:
#     st.info(f"""
# 📂 **Dataset Loaded:**
# `{data_path.split('/')[-1]}`
#     """)
# with col_info2:
#     st.info(f"""
# 📈 **Total Figures:**
# {total_figures}
#     """)
# with col_info3:
#     render_time = time.time() - st.session_state.last_time
#     st.info(f"""
# ⏱️ **Render Time:**
# {render_time:.3f} seconds
#     """)
# with col_info4:
#     try:
#         data_shape = str(data[st.session_state.current_index].shape)
#         st.info(f"""
# 📐 **Current Data Shape:**
# {data_shape}
#         """)
#     except Exception:
#         st.info("""
# 📐 **Current Data Shape:**
# N/A
#         """)

# # Navigation controls
# col1, col2, col3, col4 = st.columns([1, 1, 2, 1])
# with col1:
#     st.button("⏮ **Previous**", on_click=prev_figure,
#               disabled=(st.session_state.current_index <= 0),
#               use_container_width=True)

# with col2:
#     st.button("**Next** ⏭", on_click=next_figure,
#               disabled=(st.session_state.current_index >= total_figures - 1),
#               use_container_width=True)

# with col3:
#     st.text_input("""
# 🔢 **Go to Index**
# """, key="input_index",
#                   placeholder=f"Enter index (0-{total_figures-1})",
#                   on_change=go_to_index)

# with col4:
#     if st.button("🎲 **Random**", use_container_width=True):
#         import random
#         change_index(random.randint(0, total_figures - 1))

# # Progress indicators
# st.progress(st.session_state.current_index / (total_figures - 1))
# st.caption(f"""
# 📊 **Figure {st.session_state.current_index + 1} of {total_figures}**
# (Index: {st.session_state.current_index})
# """)

# # Display the current figure
# with st.spinner("""
# 🔄 **Generating Figure...**
# Please wait while the figure is being rendered. 🖼️
# """):
#     try:
#         current_data = data[st.session_state.current_index]
#         fig = get_figure(current_data)
#         st.plotly_chart(fig, use_container_width=True, theme="streamlit")
#     except Exception as e:
#         st.error(f"""
# ❌ **Error Displaying Figure:**
# {str(e)}
#         """)

# # Footer
# st.divider()
# st.caption("""
# Developed with ❤️ using **Streamlit** and **Plotly**.
# """)



# from math import ceil
from pathlib import Path


def chunk_prompts(prompt_material_path: Path, joiner: str, *, tasks_per_prompt: int) -> list[str]:
    """Chunk prompts based on splitting strategy.

    Args:
        prompts: List of prompts to chunk
        strategy: Either 'agent_cap' or 'task_rows'
        agent_cap: Maximum number of agents (used with 'agent_cap' strategy)
        task_rows: Number of rows/tasks per agent (used with 'task_rows' strategy)
    """
    prompts = [p for p in prompt_material_path.read_text(encoding="utf-8", errors="ignore").split(joiner) if p.strip() != ""]  # drop blank entries
    if tasks_per_prompt >= len(prompts):
        print("No need to chunk prompts, as tasks_per_prompt >= total prompts.", f"({tasks_per_prompt} >= {len(prompts)})")
        return prompts
    print(f"Chunking {len(prompts)} prompts into groups of {tasks_per_prompt} rows/tasks each.")
    grouped: list[str] = []
    for i in range(0, len(prompts), tasks_per_prompt):
        grouped.append(joiner.join(prompts[i : i + tasks_per_prompt]))
    return grouped



# """DS
# """


# from machineconfig.cluster.remote_machine import RemoteMachine, FileManager


# class Submission:
#     """Sends repo, data, root_dir and write execution command."""
#     @staticmethod
#     def transfer_sh(rm: RemoteMachine) -> None:
#         print("🚀 Using transfer.sh to send data to remote machine.")
#         cloud_download_py_script = "\n"
#         # downloading repo, this takes place prior to pyscript (otherwise, its tool late as the library is loaded at the top of the pyscript already)
#         if rm.config.copy_repo:
#             tmp_file = PathExtended(rm.job_params.repo_path_rh).expanduser().zip_n_encrypt()
#             cloud_download_py_script += f"print('Downloading `{tmp_file.collapseuser()}`.')\n"
#             cloud_download_py_script += f"P(r'{tmp_file.share_on_cloud()}').download(folder=r'{P(rm.job_params.repo_path_rh).parent}').decrypt_n_unzip()\n"
#             tmp_file.delete(sure=True)
#         for _idx, item in enumerate(rm.data):
#             cloud_download_py_script += f"P(r'{P(item).share_on_cloud()}').download(folder=r'{item.collapseuser().parent}')\n"
#         # save cloud_download_script_py
#         rm.file_manager.cloud_download_py_script_path.expanduser().write_text(cloud_download_py_script, encoding="utf-8")
#         # modify and save shell_script to including running of cloud_download_py_script before job script.
#         shell_file = rm.file_manager.shell_script_path.expanduser()
#         shell_script = shell_file.read_text(encoding="utf-8").replace("# EXTRA-PLACEHOLDER-POST", f"cd ~; python {rm.file_manager.cloud_download_py_script_path.rel2home().as_posix()}")
#         download_url = rm.file_manager.job_root.zip().share_on_cloud()
#         target = rm.file_manager.job_root.rel2home().parent.joinpath(download_url.name).as_posix()
#         tmp = f"cd ~; curl -o '{target}' '{download_url.as_url_str()}'; unzip '{target}' -d {rm.file_manager.job_root.rel2home().parent.as_posix()}"
#         shell_script = tmp + shell_script
#         with open(file=shell_file, mode='w', newline={"Windows": None, "Linux": "\n"}[rm.ssh.get_remote_machine()], encoding="utf-8") as file: file.write(shell_script)

#     @staticmethod
#     def cloud(rm: RemoteMachine) -> None:
#         cloud = rm.config.cloud_name
#         assert cloud is not None, "Cloud name is not specified in the config file. Please specify it in the config file."
#         if rm.config.copy_repo: PathExtended(rm.job_params.repo_path_rh).to_cloud(cloud=cloud, rel2home=True, zip=True, encrypt=True)
#         for x in rm.data: x.to_cloud(cloud=cloud, rel2home=True)
#         downloads = '\n'.join([f"cloud_copy {cloud}: '{a_path.collapseuser().as_posix()} -r" for a_path in rm.data])
#         if not rm.config.copy_repo: downloads += f"""\n cloud_copy {cloud}: {P(rm.job_params.repo_path_rh).collapseuser().as_posix()} -zer """
#         downloads += f"\ncloud_copy {cloud}: {rm.file_manager.job_root} -zr"
#         rm.file_manager.shell_script_path.expanduser().write_text(downloads + rm.file_manager.shell_script_path.expanduser().read_text(encoding="utf-8"), encoding='utf-8')  # newline={"Windows": None, "Linux": "\n"}[rm.ssh.get_remote_machine()]
#         PathExtended(rm.file_manager.job_root).to_cloud(cloud=cloud, zip=True, rel2home=True)

#     @staticmethod
#     def sftp(rm: RemoteMachine) -> None:
#         assert rm.ssh.sftp is not None, f"SFTP is not available for this machine `{rm.ssh}`. Consider using different `transfer_method` other than `sftp`."
#         log_path_parent = f"P(r'{FileManager.shell_script_path_log}').expanduser().parent"
#         rm.ssh.run_py(f"{log_path_parent}.mkdir(parents=True, exist_ok=True); P(r'{FileManager.shell_script_path_log}').expanduser().delete(sure=True).write_text(r'{rm.file_manager.shell_script_path.collapseuser().as_posix()}')", desc="Logging latest shell script path on remote.", verbose=False)
#         if rm.config.copy_repo: rm.ssh.copy_from_here(rm.job_params.repo_path_rh, z=True, overwrite=True)
#         for a_path in rm.data:
#             rm.ssh.copy_from_here(
#                 a_path,
#                 z=True if PathExtended(a_path).is_dir() else False,
#                 r=False,
#                 overwrite=True,
#             )
#         rm.ssh.copy_from_here(rm.file_manager.job_root, z=True)


# if __name__ == '__main__':
#     pass






from typing import Optional
from pathlib import Path
from machineconfig.utils.source_of_truth import CONFIG_ROOT, DEFAULTS_PATH

import typer


def resolve_directory(directory: Optional[str]) -> Path:
    if directory is None:
        directory = Path.cwd().as_posix()
        typer.echo(f"📁 Using directory: {directory}")
    return Path(directory).expanduser().absolute()
def git_operations(
    directory: Optional[str],
    *,
    pull: bool,
    commit: bool,
    push: bool,
    recursive: bool,
    auto_uv_sync: bool,
) -> None:
    
    repos_root = resolve_directory(directory)
    from machineconfig.scripts.python.helpers_repos.action import perform_git_operations
    from machineconfig.utils.path_extended import PathExtended
    perform_git_operations(
        repos_root=PathExtended(repos_root),
        pull=pull,
        commit=commit,
        push=push,
        recursive=recursive,
        auto_uv_sync=auto_uv_sync,
    )
def resolve_spec_path(directory: Optional[str], cloud: Optional[str]) -> Path:
    repos_root = resolve_directory(directory)
    from machineconfig.utils.path_extended import PathExtended
    if not repos_root.exists() or repos_root.name != "repos.json":
        candidate = Path(CONFIG_ROOT).joinpath("repos").joinpath(PathExtended(repos_root).rel2home()).joinpath("repos.json")
        repos_root = candidate
        if not repos_root.exists():
            cloud_name: Optional[str]
            if cloud is None:
                from machineconfig.utils.io import read_ini
                cloud_name = read_ini(DEFAULTS_PATH)["general"]["rclone_config_name"]
                typer.echo(f"⚠️ Using default cloud: {cloud_name}")
            else:
                cloud_name = cloud
            assert cloud_name is not None, (
                f"Path {repos_root} does not exist and cloud was not passed. You can't clone without one of them."
            )
            from machineconfig.utils.path_extended import PathExtended
            PathExtended(repos_root).from_cloud(cloud=cloud_name, rel2home=True)
    assert repos_root.exists() and repos_root.name == "repos.json", (
        f"Path {repos_root} does not exist and cloud was not passed. You can't clone without one of them."
    )
    return repos_root
def clone_from_specs(
    directory: Optional[str],
    cloud: Optional[str],
    *,
    checkout_branch_flag: bool,
    checkout_commit_flag: bool,
) -> None:
    
    typer.echo("\n📥 Cloning or checking out repositories...")
    spec_path = resolve_spec_path(directory, cloud)
    from machineconfig.scripts.python.helpers_repos.clone import clone_repos
    clone_repos(
        spec_path=spec_path,
        preferred_remote=None,
        checkout_branch_flag=checkout_branch_flag,
        checkout_commit_flag=checkout_commit_flag,
    )




"""
ref: https://github.com/charmbracelet/crush

"""

from pathlib import Path
import platform
from machineconfig.scripts.python.ai.solutions._shared import get_generic_instructions_path



def build_configuration(repo_root: Path) -> None:
    instructions_path = get_generic_instructions_path()
    repo_root.joinpath("CRUSH.md").write_text(data=instructions_path.read_text(encoding="utf-8"), encoding="utf-8")

    repo_settings = repo_root.joinpath(".crush.json")
    ignore_settings = repo_root.joinpath(".crushignore")
    if platform.system() == "Windows":
        global_settings = Path.home().joinpath("AppData/Local/crush/crush.json")
    else:
        global_settings = Path.home().joinpath(".config/crush/crush.json")
    
    _ = repo_settings, ignore_settings, global_settings  # to avoid unused variable warnings



"""Mac setup module for machineconfig.

This module provides setup script locations and configurations for macOS systems.
It mirrors the layout used by the Linux setup module and exposes Path objects
pointing at common scripts (so other code can import these paths).
"""

from pathlib import Path

# Path to main installer script for macOS
APPS = Path(__file__).parent.joinpath("apps.sh")
# Optional helper scripts (may or may not exist)
UV = Path(__file__).parent.joinpath("uv.sh")
# Path to macOS SSH helper
SSH_SETUP = Path(__file__).parent.joinpath("ssh/openssh_setup.sh")

__all__ = ["APPS", "UV", "SSH_SETUP"]



# """
# Distributed Computing
# """


# from typing import Optional, Any, Callable, Union
# from math import ceil, floor
# from enum import Enum
# from dataclasses import dataclass

# from machineconfig.utils.utils2 import randstr
# from machineconfig.utils.io_save import save_pickle
# from machineconfig.cluster.remote_machine import RemoteMachine, RemoteMachineConfig, WorkloadParams, LAUNCH_METHOD
# from rich.console import Console
# from machineconfig.utils.utils2 import pprint
# import io
# # from platform import system
# # import time
# # from rich.progress import track


# console = Console()


# class LoadCriterion(Enum):
#     cpu = "cpu"
#     ram = "ram"
#     product = "cpu * ram"
#     cpu_norm = "cpu_norm"
#     ram_norm = "ram_norm"
#     product_norm = "cpu_norm * ram_norm"


# @dataclass
# class MachineSpecs:
#     cpu: float
#     ram: float
#     product: float
#     cpu_norm: float
#     ram_norm: float
#     product_norm: float
#     @staticmethod
#     def get_this_machine_specs():
#         cpu, ram = psutil.cpu_count(), psutil.virtual_memory().total / 2 ** 30
#         assert cpu is not None
#         return MachineSpecs(cpu=cpu, ram=ram, product=cpu * ram, cpu_norm=cpu, ram_norm=ram, product_norm=cpu * ram)


# class ThreadLoadCalculator:
#     """relies on relative values to a referenc machine specs.
#     Runs multiple instances of code per machine. Useful if code doesn't run faster with more resources avaliable.
#     equal distribution across instances of one machine"""
#     def __init__(self, num_jobs: Optional[int] = None, load_criterion: LoadCriterion = LoadCriterion.cpu, reference_specs: Optional[MachineSpecs] = None):
#         self.num_jobs = num_jobs
#         self.load_criterion = load_criterion
#         self.reference_specs: MachineSpecs = MachineSpecs.get_this_machine_specs() if reference_specs is None else reference_specs
#     def __getstate__(self): return self.__dict__
#     def __setstate__(self, state: dict[str, Any]): self.__dict__.update(state)
#     def get_num_threads(self, machine_specs: MachineSpecs) -> int:
#         if self.num_jobs is None: return 1
#         res = int(floor(self.num_jobs * (machine_specs.__dict__[self.load_criterion.name] / self.reference_specs.__dict__[self.load_criterion.name])))
#         return 1 if res == 0 else res


# class MachineLoadCalculator:
#     def __init__(self, max_num: int = 1000, load_criterion: LoadCriterion = LoadCriterion.product, load_ratios_repr: str = ""):
#         self.load_ratios: list[float] = []
#         self.load_ratios_repr = load_ratios_repr
#         self.max_num: int = max_num
#         self.load_criterion = load_criterion
#     def __getstate__(self) -> dict[str, Any]: return self.__dict__
#     def __setstate__(self, d: dict[str, Any]) -> None: self.__dict__.update(d)
#     def get_workload_params(self, machines_specs: list[MachineSpecs], threads_per_machine: list[int]) -> list[WorkloadParams]:
#         """Note: like thread divider in parallelize function, the behaviour is to include the edge cases on both ends of subsequent intervals."""
#         tmp: list[WorkloadParams] = []
#         idx_so_far = 0
#         for machine_index, (machine_specs, a_threads_per_machine) in enumerate(zip(machines_specs, threads_per_machine)):
#             load_value = machine_specs.__dict__[self.load_criterion.name]
#             self.load_ratios.append(load_value)
#             idx1 = idx_so_far
#             idx2 = self.max_num if machine_index == len(threads_per_machine) - 1 else (floor(load_value * self.max_num) + idx1)
#             if idx2 > self.max_num:
#                 print(machines_specs, '\n\n', threads_per_machine)
#                 print(f"All values: {tmp=}, {idx_so_far=}, {idx1=}, {idx2=}, {self.max_num=}, {a_threads_per_machine=}, {machine_index=}, {machine_specs=}, {load_value=}, {self.load_ratios=}, {self.load_ratios_repr=}")
#                 raise ValueError(f"idx2 ({idx2}) > max_num ({self.max_num})")
#             idx_so_far = idx2
#             tmp.append(WorkloadParams(idx_start=idx1, idx_end=idx2, idx_max=self.max_num, jobs=a_threads_per_machine))
#         return tmp


# class Cluster:
#     def __getstate__(self) -> dict[str, Any]:
#         state = self.__dict__
#         state["func"] = None
#         return state
#     def __setstate__(self, state: dict[str, Any]) -> None: self.__dict__.update(state)
#     def save(self) -> P:
#         path = self.root_dir.joinpath("cluster.Cluster.pkl")
#         save_pickle(obj=self.__getstate__(), path=path)
#         return path
#     @staticmethod
#     def load(job_id: str, base: Optional[str] = None) -> 'Cluster': return Cluster.get_cluster_path(job_id=job_id, base=base).joinpath("cluster.Cluster.pkl").readit()
#     @staticmethod
#     def get_cluster_path(job_id: str, base: Union[str, P, None] = None):
#         if base is None: base_obj = PathExtended.home().joinpath("tmp_results/remote_machines")
#         else: base_obj = PathExtended(base)
#         return base_obj.joinpath(f"job_id__{job_id}")
#     def __init__(self,
#                  func: Callable[..., Any],
#                  ssh_params: list[dict[str, Any]],
#                  remote_machine_config: RemoteMachineConfig,
#                  func_kwargs: Optional[dict[str, Any]] = None,
#                  # workload_params: list[WorkloadParams] or None = None,
#                  thread_load_calc: Optional[ThreadLoadCalculator] = None,
#                  # machine_load_calc=None,
#                  ditch_unavailable_machines: bool = False,
#                  description: str = "",
#                  job_id: Optional[str] = None,
#                  base_dir: Union[str, P, None] = None):
#         self.job_id = job_id or randstr(noun=True)
#         self.root_dir = self.get_cluster_path(self.job_id, base=base_dir)
#         self.results_downloaded = False

#         self.thread_load_calc: ThreadLoadCalculator = thread_load_calc or ThreadLoadCalculator()
#         self.machine_load_calc: MachineLoadCalculator = MachineLoadCalculator(load_criterion=LoadCriterion[self.thread_load_calc.load_criterion.name + "_norm"], )

#         sshz: list[SSH] = []
#         for an_ssh_params in ssh_params:
#             try:
#                 tmp = SSH(**an_ssh_params)
#                 sshz.append(tmp)
#             except Exception as ex:
#                 print(f"Couldn't connect to {an_ssh_params}")
#                 if ditch_unavailable_machines: continue
#                 else: raise Exception(f"Couldn't connect to {an_ssh_params}") from ex  # type: ignore # pylint: disable=W0719

#         # lists of similar length:
#         self.sshz: list[SSH] = sshz
#         self.machines: list[RemoteMachine] = []
#         self.machines_specs: list[MachineSpecs] = []
#         self.threads_per_machine: list[int] = []
#         self.remote_machine_kwargs: RemoteMachineConfig = remote_machine_config
#         self.workload_params: list[WorkloadParams] = []

#         self.description: str = description
#         self.func = func
#         self.func_kwargs = func_kwargs if func_kwargs is not None else {}

#         # fire options
#         self.machines_per_tab: int = 1
#         self.window_number: int = 2

#     def __repr__(self): return "Cluster with following machines:\n" + "\n".join([repr(item) for item in (self.machines if self.machines else self.sshz)])
#     def print_func_kwargs(self):
#         print("\n" * 2)
#         console.rule(title="kwargs of functions to be run on machines")
#         for an_ssh, a_kwarg in zip(self.sshz, self.workload_params):
#             pprint(a_kwarg.__dict__, an_ssh.get_remote_repr())
#     def print_commands(self, launch_method: LAUNCH_METHOD):
#         print("\n" * 2)
#         console.rule(title="Commands to run on each machine:")
#         for machine in self.machines:
#             print(f"{repr(machine)} ==> {machine.file_manager.get_fire_command(launch_method=launch_method)}")

#     def generate_standard_kwargs(self) -> None:
#         if self.workload_params:
#             self.print_func_kwargs()
#             print(self.workload_params, len(self.workload_params), type(self.workload_params))
#             print("workload_params is not None, so not generating standard kwargs")
#             return None
#         cpus: list[float] = []
#         rams: list[float] = []
#         for an_ssh in self.sshz:
#             res = an_ssh.run_py("import psutil; print(psutil.cpu_count(), psutil.virtual_memory().total)", verbose=False).op
#             try: cpus.append(int(res.split(' ')[0]))
#             except ValueError as ve:
#                 print(f"Couldn't get cpu count from {an_ssh}")
#                 raise ValueError(f"Couldn't get cpu count from {an_ssh.get_remote_repr()}") from ve
#             rams.append(ceil(int(res.split(' ')[1]) / 2 ** 30))
#         total_cpu = np.array(cpus).sum()
#         total_ram = np.array(rams).sum()
#         total_product = (np.array(cpus) * np.array(rams)).sum()

#         self.machines_specs = [MachineSpecs(cpu=a_cpu, ram=a_ram, product=a_cpu * a_ram, cpu_norm=a_cpu / total_cpu, ram_norm=a_ram / total_ram, product_norm=a_cpu * a_ram / total_product) for a_cpu, a_ram in zip(cpus, rams)]
#         self.threads_per_machine = [self.thread_load_calc.get_num_threads(machine_specs=machine_specs) for machine_specs in self.machines_specs]
#         self.workload_params = self.machine_load_calc.get_workload_params(machines_specs=self.machines_specs, threads_per_machine=self.threads_per_machine)
#         self.print_func_kwargs()

#     def viz_load_ratios(self) -> None:
#         if not self.workload_params: raise RuntimeError("func_kwargs_list is None. You need to run generate_standard_kwargs() first.")
#         import plottext
#         plt.simple_multiple_bar(names, [[machine_specs.cpu for machine_specs in self.machines_specs], [machine_specs.ram for machine_specs in self.machines_specs]], title="Resources per machine", labels=["#cpu threads", "memory size"])
#         plt.show()
#         print("")
#         plt.simple_bar(names, self.machine_load_calc.load_ratios, width=100, title=f"Load distribution for machines using criterion `{self.machine_load_calc.load_criterion}`")
#         plt.show()

#         # Capture load ratios as string
#         buffer = io.StringIO()
#         from rich import inspect
#         Console(file=buffer, width=75).print(inspect(dict(zip(names, L((np.array(self.machine_load_calc.load_ratios) * 100).round(1)).apply(lambda x: f"{int(x)}%"))), value=False, docs=False, dunder=False, sort=False))
#         tmp = buffer.getvalue()
#         assert isinstance(tmp, str)
#         self.machine_load_calc.load_ratios_repr = tmp
#         print(self.machine_load_calc.load_ratios_repr)
#         # self.workload_params.
#         print("\n")

#     def submit(self) -> None:
#         if not self.workload_params: raise RuntimeError("You need to generate standard kwargs first.")
#         for idx, (a_workload_params, an_ssh) in enumerate(zip(self.workload_params, self.sshz)):
#             desc = self.description + f"\nLoad Ratios on machines:\n{self.machine_load_calc.load_ratios_repr}"
#             # if self.remote_machine_kwargs is not None:
#             config = self.remote_machine_kwargs
#             config.__dict__.update(dict(description=desc, job_id=self.job_id + f"_{idx}", base_dir=self.root_dir, workload_params=a_workload_params, ssh_obj=an_ssh))
#             # else: config = RemoteMachineConfig(description=desc, job_id=self.job_id + f"_{idx}", base_dir=self.root_dir.as_posix(), workload_params=a_workload_params, ssh_obj=an_ssh)
#             m = RemoteMachine(func=self.func, func_kwargs=self.func_kwargs, config=config)
#             m.generate_scripts()
#             m.submit()
#             self.machines.append(m)
#         try: self.save()
#         except Exception as re:
#             print(re)
#             print("Couldn't pickle cluster object")
#         # self.print_commands()

#     def open_mux(self, machines_per_tab: int = 1, window_number: Optional[int] = None):
#         self.machines_per_tab = machines_per_tab
#         self.window_number = window_number if window_number is not None else 0  # randstr(length=3, lower=False, upper=False)
#         cmd = f"wt -w {self.window_number} "
#         for idx, m in enumerate(self.machines):

#             sub_cmd = m.get_session_manager().get_new_session_ssh_command(ssh=m.ssh, sess_name=m.job_params.session_name)
#             if idx == 0: cmd += f""" new-tab --title '{str(m.ssh.hostname) + str(idx)}' pwsh -Command "{sub_cmd}" `;"""  # avoid new tabs despite being even index
#             elif idx % self.machines_per_tab == 0: cmd += f""" new-tab --title {str(m.ssh.hostname) + str(idx)} pwsh -Command "{sub_cmd}" `;"""
#             else: cmd += f""" split-pane --horizontal --size {1 / self.machines_per_tab} pwsh -Command "{sub_cmd}" `;"""

#         print("Terminal launch command:\n", cmd)
#         if cmd.endswith("`;"): cmd = cmd[:-2]
#         Terminal().run_async(*cmd.replace("`;", ";").split(" "))  # `; only for powershell, cmd is okay for ; as it is not a special character
#         rm_last = self.machines[-1]
#         rm_last.get_session_manager().asssert_session_started(ssh=rm_last.ssh, sess_name=rm_last.job_params.session_name)

#     def fire(self, machines_per_tab: int = 1, window_number: Optional[int] = None, run: bool = False):
#         self.open_mux(machines_per_tab=machines_per_tab, window_number=window_number)
#         for m in self.machines:
#             m.fire(run=run, open_console=False)

#     def run(self, run: bool = False, machines_per_tab: int = 1, window_number: Optional[int] = None):
#         self.generate_standard_kwargs()
#         self.viz_load_ratios()
#         print(self)
#         self.submit()
#         self.fire(run=run, machines_per_tab=machines_per_tab, window_number=window_number)
#         self.save()
#         return self

#     def check_job_status(self) -> None: L(self.machines).apply(lambda machine: machine.check_job_status())
#     def download_results(self):
#         if self.results_downloaded:
#             print(f"All results downloaded to {self.root_dir} 🤗")
#             return True
#         for idx, a_m in enumerate(self.machines):
#             _ = idx
#             if a_m.results_path is None:
#                 print(f"Results are not ready for machine {a_m}.")
#                 print("Try to run `.check_job_status()` to check if the job is done and obtain results path.")
#                 continue
#             # results_folder = PathExtended(a_m.results_path).expanduser()
#             if a_m.results_downloaded is False:
#                 print("\n")
#                 console.rule(f"Downloading results from {a_m}")
#                 print("\n")
#                 a_m.download_results(target=None)  # TODO another way of resolve multiple machines issue is to create a directory at downlaod_results time.
#         if L(self.machines).results_downloaded.to_numpy().sum() == len(self.machines):
#             print(f"All results downloaded to {self.root_dir} 🤗")
#             self.results_downloaded = True


# if __name__ == '__main__':
#     pass



"""
Choose a theme for Wezterm
"""

from machineconfig.utils.options import choose_from_options
from machineconfig.utils.path_extended import PathExtended
from typing import Any
import time
from rich.panel import Panel
from rich.console import Console
import curses

console = Console()


schemes_list = [
    "Pro",
    "Spiderman",
    "shades-of-purple",
    "synthwave",
    "Symfonic",
    "PaulMillr",
    "Neon",
    "LiquidCarbonTransparentInverse",
    "Laser",
    "IR_Black",
    "Hurtado",
    "Homebrew",
    "Hipster Green",
    "Firefly Traditional",
    "Elementary",
    "deep",
    "Dark Pastel",
    "Bright Lights",
    "Adventure",
    "Nancy (terminal.sexy)",
    "Bim (Gogh)",
    "BlueDolphin",
    "Borland",
    "Grass (Gogh)",
    "Greenscreen (light) (terminal.sexy)",
    "Grayscale (dark) (terminal.sexy)",
]


def main2():
    console.print(Panel("🎨 WezTerm Theme Selector", title_align="left", border_style="green"))
    option = choose_from_options(multi=False, options=schemes_list, header="Choose a theme for Wezterm", fzf=True, msg="Use arrow keys to navigate, Enter to select a theme")
    set_theme(option)
    print(f"✅ Theme set to: {option}")


def set_theme(theme: str):
    print(f"🔄 Setting WezTerm theme to: {theme}")
    txt_lines = PathExtended.home().joinpath(".config/wezterm/wezterm.lua").expanduser().read_text(encoding="utf-8").splitlines()
    res_lines = []
    for line in txt_lines:
        if "config.color_scheme = " in line:
            res_lines.append(f"config.color_scheme = '{theme}'")
        else:
            res_lines.append(line)
    PathExtended.home().joinpath(".config/wezterm/wezterm.lua").expanduser().write_text("\n".join(res_lines), encoding="utf-8")
    time.sleep(0.1)
    print("💾 Configuration saved")


def main():
    console.print(Panel("🎨 WezTerm Theme Selector - Interactive Mode", title_align="left", border_style="blue"))
    print("""
📝 Use arrow keys to navigate, Enter to select a theme
""")
    curses.wrapper(accessory)
    console.print(Panel("✅ Theme selection completed", title_align="left", border_style="green"))


def accessory(stdscr: Any):
    options = schemes_list
    current_option = 0
    page_size = stdscr.getmaxyx()[0] - 1  # curses.LINES - 1  # Number of lines in the terminal, -1 for status line

    while True:
        stdscr.clear()

        # Calculate start and end indices for options
        start_index = (current_option // page_size) * page_size
        end_index = start_index + page_size

        # Display options
        for i, option in enumerate(options[start_index:end_index]):
            if start_index + i == current_option:
                stdscr.addstr(i, 0, f"➤ {option}", curses.A_REVERSE)  # Highlighted with arrow
            else:
                stdscr.addstr(i, 0, f"  {option}")

        # Display status line
        status_line = f"🔍 Theme {current_option + 1}/{len(options)} | ↑/↓: Navigate | Enter: Select"
        stdscr.addstr(page_size, 0, status_line)

        # Get key press
        key = stdscr.getch()

        # Handle key press
        if key == curses.KEY_UP and current_option > 0:
            current_option -= 1
        elif key == curses.KEY_DOWN and current_option < len(options) - 1:
            current_option += 1
        elif key == ord("\n"):  # Enter key
            break  # Exit the loop
        set_theme(options[current_option])



"""
Helper modules for the devops navigator TUI application.
"""

from machineconfig.scripts.python.helpers_navigator.data_models import CommandInfo, ArgumentInfo
from machineconfig.scripts.python.helpers_navigator.command_builder import CommandBuilderScreen
from machineconfig.scripts.python.helpers_navigator.command_tree import CommandTree
from machineconfig.scripts.python.helpers_navigator.command_detail import CommandDetail
from machineconfig.scripts.python.helpers_navigator.search_bar import SearchBar
from machineconfig.scripts.python.helpers_navigator.main_app import CommandNavigatorApp

__all__ = [
    "CommandInfo",
    "ArgumentInfo",
    "CommandBuilderScreen",
    "CommandTree",
    "CommandDetail",
    "SearchBar",
    "CommandNavigatorApp",
]


"""Devops Devapps Install"""

from machineconfig.utils.installer import dynamically_extract_installers_system_groups_from_scripts
import typer
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from typing import Optional, cast, get_args, Annotated
from machineconfig.jobs.installer.package_groups import PACKAGE_GROUPS, PACKAGE_GROUP2NAMES

console = Console()


def _handle_installer_not_found(search_term: str, all_names: list[str]) -> None:  # type: ignore
    """Handle installer not found with friendly suggestions using fuzzy matching."""
    from difflib import get_close_matches
    close_matches = get_close_matches(search_term, all_names, n=5, cutoff=0.4)
    console.print(f"\n❌ '[red]{search_term}[/red]' was not found.", style="bold")
    if close_matches:
        console.print("🤔 Did you mean one of these?", style="yellow")
        table = Table(show_header=False, box=None, pad_edge=False)
        for i, match in enumerate(close_matches, 1):
            table.add_row(f"[cyan]{i}.[/cyan]", f"[green]{match}[/green]")
        console.print(table)
    else:
        console.print("📋 Here are some available options:", style="blue")
        # Show first 10 installers as examples
        if len(all_names) > 10:
            sample_names = all_names[:10]
        else:
            sample_names = all_names
        table = Table(show_header=False, box=None, pad_edge=False)
        for i, name in enumerate(sample_names, 1):
            table.add_row(f"[cyan]{i}.[/cyan]", f"[green]{name}[/green]")
        console.print(table)
        if len(all_names) > 10:
            console.print(f"   [dim]... and {len(all_names) - 10} more[/dim]")

    panel = Panel(f"[bold blue]💡 Use 'ia' to interactively browse all available installers.[/bold blue]\n[bold blue]💡 Use one of the categories: {list(get_args(PACKAGE_GROUPS))}[/bold blue]", title="[yellow]Helpful Tips[/yellow]", border_style="yellow")
    console.print(panel)


def main_with_parser():
    import typer
    app = typer.Typer()
    app.command()(main)
    app()


def main(
    which: Annotated[Optional[str], typer.Argument(..., help="Comma-separated list of program/groups names to install (if --group flag is set).")] = None,
    group: Annotated[bool, typer.Option(..., "--group", "-g", help="Treat 'which' as a group name. A group is bundle of apps.")] = False,
    interactive: Annotated[bool, typer.Option(..., "--interactive", "-ia", help="Interactive selection of programs to install.")] = False,
) -> None:
    if interactive:
        return install_interactively()
    if which is not None:
        if group:
            for a_group in [x.strip() for x in which.split(",") if x.strip() != ""]:
                return install_group(package_group=a_group)
        else:
            return install_clis(clis_names=[x.strip() for x in which.split(",") if x.strip() != ""])
    else:
        if group:
            typer.echo("❌ You must provide a group name when using the --group/-g option.")
            res = get_static_groups_combined_with_dynamic_groups_extracted()
            console.print("[bold blue]Here are the available groups:[/bold blue]")
            table = Table(show_header=True, header_style="bold magenta")
            table.add_column("Group", style="cyan", no_wrap=True)
            table.add_column("AppsBundled", style="green", overflow="fold")
            for display, group_name in res.items():
                # Parse display
                if "   --   " in display:
                    group_part, items_part = display.split("   --   ", 1)
                    group_name_parsed = group_part.replace("📦 ", "").strip()
                    items_str = items_part.strip()
                else:
                    group_name_parsed = display
                    items_str = group_name
                table.add_row(group_name_parsed, items_str)
            console.print(table)
            raise typer.Exit(1)
    typer.echo("❌ You must provide either a program name/group name, or use --interactive/-ia option.")
    import click
    ctx = click.get_current_context()
    typer.echo(ctx.get_help())
    raise typer.Exit(1)


def get_static_groups_combined_with_dynamic_groups_extracted():
    # Build category options and maintain a mapping from display text to actual category name
    category_display_to_name: dict[str, str] = {}
    for group_name, group_values in PACKAGE_GROUP2NAMES.items():
        display = f"📦 {group_name:<20}" + "   --   " + f"{'|'.join(group_values):<60}"
        category_display_to_name[display] = group_name
    options_system = dynamically_extract_installers_system_groups_from_scripts()
    for item in options_system:
        display = f"📦 {item['appName']:<20}   --   {item['doc']:<60}"
        category_display_to_name[display] = item['appName']
    return category_display_to_name


def install_interactively():
    from machineconfig.utils.options import choose_from_options
    from machineconfig.utils.schemas.installer.installer_types import get_normalized_arch, get_os_name
    from machineconfig.utils.installer import get_installers
    from machineconfig.utils.installer_utils.installer_class import Installer
    installers = get_installers(os=get_os_name(), arch=get_normalized_arch(), which_cats=None)
    installer_options = []
    for x in installers:
        installer_options.append(Installer(installer_data=x).get_description())

    category_display_to_name = get_static_groups_combined_with_dynamic_groups_extracted()
    options = list(category_display_to_name.keys()) + ["─" * 50] + installer_options
    program_names = choose_from_options(multi=True, msg="Categories are prefixed with 📦", options=options, header="🚀 CHOOSE DEV APP OR CATEGORY", default="📦 essentials", fzf=True)
    installation_messages: list[str] = []
    for _an_idx, a_program_name in enumerate(program_names):
        if a_program_name.startswith("─"):  # 50 dashes separator
            continue
        if a_program_name.startswith("📦 "):
            category_name = category_display_to_name.get(a_program_name)
            if category_name:
                install_group(package_group=cast(PACKAGE_GROUPS, category_name))
        else:
            installer_idx = installer_options.index(a_program_name)
            an_installer_data = installers[installer_idx]
            status_message = Installer(an_installer_data).install_robust(version=None)  # finish the task - this returns a status message, not a command
            installation_messages.append(status_message)
    if installation_messages:
        panel = Panel("\n".join([f"[blue]• {message}[/blue]" for message in installation_messages]), title="[bold green]📊 Installation Summary[/bold green]", border_style="green", padding=(1, 2))
        console.print(panel)


def install_group(package_group: str):
    from machineconfig.utils.installer import get_installers, install_bulk
    from machineconfig.utils.schemas.installer.installer_types import get_normalized_arch, get_os_name
    if package_group in PACKAGE_GROUP2NAMES:
        panel = Panel(f"[bold yellow]Installing programs from category: [green]{package_group}[/green][/bold yellow]", title="[bold blue]📦 Category Installation[/bold blue]", border_style="blue", padding=(1, 2))
        console.print(panel)
        installers_ = get_installers(os=get_os_name(), arch=get_normalized_arch(), which_cats=[package_group])
        install_bulk(installers_data=installers_)
        return
    options_system = dynamically_extract_installers_system_groups_from_scripts()
    from machineconfig.utils.schemas.installer.installer_types import get_normalized_arch, get_os_name
    from machineconfig.utils.code import run_shell_script
    for an_item in options_system:
        if an_item["appName"] == package_group:
            panel = Panel(f"[bold yellow]Installing programs from category: [green]{package_group}[/green][/bold yellow]", title="[bold blue]📦 Category Installation[/bold blue]", border_style="blue", padding=(1, 2))
            console.print(panel)
            program = an_item["fileNamePattern"][get_normalized_arch()][get_os_name()]
            if program is not None:
                run_shell_script(program)
                break
    else:
        console.print(f"❌ [red]Group '{package_group}' not found.[/red]", style="bold")
        _handle_installer_not_found(package_group, all_names=list(PACKAGE_GROUP2NAMES.keys()) + [item['appName'] for item in options_system])


def choose_from_system_package_groups(options_system: dict[str, tuple[str, str]]) -> str:
    from machineconfig.utils.options import choose_from_options
    display_options = []
    for group_name, (description, _) in options_system.items():
        if description:
            display_options.append(f"{group_name:<20} - {description}")
        else:
            display_options.append(group_name)
    program_names = choose_from_options(multi=True, msg="", options=sorted(display_options), header="🚀 CHOOSE DEV APP", fzf=True)
    program = ""
    for display_name in program_names:
        # Extract the actual group name (everything before " - " if present)
        group_name = display_name.split(" - ")[0].strip() if " - " in display_name else display_name.strip()
        console.print(f"\n[bold cyan]⚙️  Installing: [yellow]{group_name}[/yellow][/bold cyan]", style="bold")
        _, sub_program = options_system[group_name]  # Extract content from tuple
        if sub_program.startswith("#winget"):
            sub_program = sub_program[1:]
        program += "\n" + sub_program
    return program


def install_clis(clis_names: list[str]):
    from machineconfig.utils.schemas.installer.installer_types import get_normalized_arch, get_os_name
    from machineconfig.utils.installer import get_installers
    from machineconfig.utils.installer_utils.installer_class import Installer
    total_messages: list[str] = []
    for a_which in clis_names:
        all_installers = get_installers(os=get_os_name(), arch=get_normalized_arch(), which_cats=None)
        selected_installer = None
        for installer in all_installers:
            app_name = installer["appName"]
            if app_name.lower() == a_which.lower():
                selected_installer = installer
                break
        if selected_installer is None:
            _handle_installer_not_found(a_which, all_names=[inst["appName"] for inst in all_installers])
            return None
        message = Installer(selected_installer).install_robust(version=None)  # finish the task
        total_messages.append(message)
    if total_messages:
        console.print("\n[bold green]📊 Installation Results:[/bold green]")
        for a_message in total_messages:
            console.print(f"[blue]• {a_message}[/blue]")
    return None


def install_if_missing(which: str):
    from machineconfig.utils.installer_utils.installer_abc import check_tool_exists
    exists = check_tool_exists(which)
    if exists:
        print(f"✅ {which} is already installed.")
        return
    print(f"⏳ {which} not found. Installing...")
    from machineconfig.utils.installer_utils.installer import main
    main(which=which, interactive=False)


if __name__ == "__main__":
    from machineconfig.utils.schemas.installer.installer_types import InstallerData
    from machineconfig.utils.installer_utils.installer_class import Installer

    _ = InstallerData, Installer
    pass



"""brave installer"""

import platform
import subprocess
from typing import Optional
from rich import box
from rich.console import Console
from rich.panel import Panel
from machineconfig.utils.schemas.installer.installer_types import InstallerData



def main(installer_data: InstallerData, version: Optional[str]) -> None:
    console = Console()
    _ = installer_data
    console.print(
        Panel.fit(
            "\n".join([f"💻 Platform: {platform.system()}", f"🔄 Version: {'latest' if version is None else version}"]),
            title="🦁 Brave Browser Installer",
            border_style="orange1",
            box=box.ROUNDED,
        )
    )

    _ = version
    if platform.system() == "Windows":
        console.print("🪟 Installing Brave Browser on Windows using winget...", style="bold")
        program = """

winget install --Name "Brave Browser" --Id Brave.Brave --source winget --accept-package-agreements --accept-source-agreements

"""
    elif platform.system() in ["Linux", "Darwin"]:
        system_name = "Linux" if platform.system() == "Linux" else "macOS"
        console.print(f"🐧 Installing Brave Browser on {system_name}...", style="bold")
        import machineconfig.jobs.installer as module
        from pathlib import Path

        if platform.system() == "Linux":
            program = Path(module.__file__).parent.joinpath("linux_scripts/brave.sh").read_text(encoding="utf-8")
        else:  # Darwin/macOS
            program = "brew install --cask brave-browser"
    else:
        error_msg = f"Unsupported platform: {platform.system()}"
        console.print(
            Panel.fit(
                "\n".join([error_msg]),
                title="❌ Error",
                subtitle="⚠️ Unsupported platform",
                border_style="red",
                box=box.ROUNDED,
            )
        )
        raise NotImplementedError(error_msg)

    console.print(
        Panel.fit(
            "\n".join(
                [
                    "🔒 Built-in ad blocking",
                    "🛡️ Privacy-focused browsing",
                    "💨 Faster page loading",
                    "🪙 Optional crypto rewards",
                ]
            ),
            title="ℹ️  Brave Browser Features",
            border_style="magenta",
            box=box.ROUNDED,
        )
    )

    console.print("🔄 EXECUTING | Running Brave Browser installation...", style="bold yellow")
    try:
        subprocess.run(program, shell=True, text=True, check=True)
        console.print("✅ Brave Browser installation completed successfully", style="bold green")
    except subprocess.CalledProcessError as e:
        console.print(f"❌ Installation failed with exit code {e.returncode}", style="bold red")
        raise


if __name__ == "__main__":
    pass




import glob
import os
import platform
import random
from pathlib import Path
from rich import pretty
from rich.console import Console


def print_header():
    console = Console()
    pretty.install()

    # Environment Information Panel
    from rich.panel import Panel
    from rich.table import Table

    table = Table(show_header=False, show_edge=False, pad_edge=False)
    table.add_column("Label", style="cyan", no_wrap=True)
    table.add_column("Value", style="white")

    table.add_row("Python Version", platform.python_version())
    table.add_row("Operating System", platform.system())
    table.add_row("Virtual Environment", os.getenv('VIRTUAL_ENV', 'None'))
    table.add_row("Running @", str(Path.cwd()))

    from machineconfig.utils.installer import get_machineconfig_version

    console.print(Panel(table, title=f"[bold blue]✨ 🐊 Machineconfig Shell {get_machineconfig_version()} ✨ Made with 🐍 | Built with ❤️[/bold blue]", border_style="blue"))
def print_logo(logo: str):
    from machineconfig.utils.files.ascii_art import font_box_color, character_color, character_or_box_color
    if platform.system() == "Windows":
        _1x = Path.home().joinpath(r"AppData/Roaming/npm/figlet").exists()
        _2x = Path.home().joinpath(r"AppData/Roaming/npm/lolcatjs").exists()
        _3x = Path.home().joinpath(r"AppData/Local/Microsoft/WindowsApps/boxes.exe").exists()
        if _1x and _2x and _3x:
            if random.choice([True, True, False]): font_box_color(logo)
            else: character_color(logo)
        else:
            # print("\n" + "🚫 " + "-" * 70 + " 🚫")
            # print("🔍 Missing ASCII art dependencies. Install with: iwr bit.ly/cfgasciiartwindows | iex")
            # print("🚫 " + "-" * 70 + " 🚫\n")
            _default_art = Path(random.choice(glob.glob(str(Path(__file__).parent.joinpath("art", "*")))))
            print(_default_art.read_text())
    elif platform.system() in ["Linux", "Darwin"]:  # Explicitly handle both Linux and macOS
        from machineconfig.utils.installer_utils.installer_abc import is_executable_in_path
        avail_cowsay = is_executable_in_path("cowsay")
        avail_lolcat = is_executable_in_path("lolcat")
        avail_boxes = is_executable_in_path("boxes")
        avail_figlet = is_executable_in_path("figlet")
        if avail_cowsay and avail_lolcat and avail_boxes and avail_figlet:
            # _dynamic_art = random.choice([True, True, True, True, False])
            # if _dynamic_art: character_or_box_color(logo=logo)
            # else:
            #     print(Path(random.choice(glob.glob(str(Path(__file__).parent.joinpath("art", "*"))))).read_text())
            character_or_box_color(logo=logo)
        else:
            print("\n" + "🚫 " + "-" * 70 + " 🚫")
            install_cmd = "devops install --group TerminalEyeCandy" if platform.system() == "Linux" else "brew install cowsay lolcat boxes figlet"
            print(f"🔍 Missing ASCII art dependencies. Install with: {install_cmd}")
            print("🚫 " + "-" * 70 + " 🚫\n")
            _default_art = Path(random.choice(glob.glob(str(Path(__file__).parent.joinpath("art", "*")))))
            print(_default_art.read_text())
    else:
        print(f"⚠️ Platform {platform.system()} not supported for ASCII art. Using default art.")
        _default_art = Path(random.choice(glob.glob(str(Path(__file__).parent.joinpath("art", "*")))))
        print(_default_art.read_text())




"""
Minimalist programs that only print scripts without frills so it can be sourced by by shell.
"""


import typer
import platform


def define_scripts():
    if platform.system() != "Linux":
        raise RuntimeError("This command is only supported on Linux systems.")
    from machineconfig.setup_linux import INTERACTIVE as script_path
    script = script_path.read_text(encoding="utf-8")
    print(script)


def get_app():
    app = typer.Typer(add_completion=False, no_args_is_help=True)
    app.command(name="scripts", help="define all scripts", no_args_is_help=False)(define_scripts)
    return app

def main():
    # return app
    app = get_app()
    app()


# if __name__ == "__main__":
#     main()




from typing import TYPE_CHECKING, Annotated
from git import Repo
from collections import defaultdict
from datetime import datetime

from pathlib import Path
from rich.progress import track
import typer


if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional, Union
    import polars as pl


app = typer.Typer()


def count_lines_in_commit(commit: "Any") -> int:
    _total_lines = 0
    for _file in commit.stats.files:
        if str(_file).endswith(".py"):
            _blob = commit.tree / _file
            _total_lines += len(_blob.data_stream.read().decode("utf-8").splitlines())
    return _total_lines


def count_historical_loc(repo_path: str) -> int:
    repo = Repo(repo_path)
    file_line_counts: "Dict[str, int]" = defaultdict(int)
    total_commits: int = sum(1 for _ in repo.iter_commits())
    print(f"Total commits to process: {total_commits}")
    for i, commit in enumerate(repo.iter_commits(), 1):
        if i % 100 == 0 or i == total_commits:
            print(f"Processing commit {i}/{total_commits} ({i / total_commits:.1%})")
        try:
            # Handle initial commits that have no parents
            if not commit.parents:
                # For initial commit, count all lines in Python files
                for file in commit.stats.files:
                    if str(file).endswith(".py"):
                        file_line_counts[str(file)] += commit.stats.files[file]["insertions"]
            else:
                # For commits with parents, use stats
                for file in commit.stats.files:
                    if str(file).endswith(".py"):
                        file_line_counts[str(file)] += commit.stats.files[file]["insertions"]
        except Exception:
            # If stats fail (e.g., corrupted parent), skip this commit
            print(f"Warning: Could not get stats for commit {commit.hexsha[:8]}, skipping")
            continue

    print(f"\nProcessed files: {len(file_line_counts)}")
    return sum(file_line_counts.values())

def count_python_lines(commit: "Any") -> int:
    """Count total lines in Python files for a specific commit"""
    total_lines = 0
    try:
        for blob in commit.tree.traverse():
            if blob.path.endswith(".py"):
                try:
                    content = blob.data_stream.read().decode("utf-8")
                    total_lines += len(content.splitlines())
                except Exception as _e:
                    continue
    except Exception as _e:
        return 0
    return total_lines
def get_default_branch(repo: Repo) -> str:
    """Get the default branch name of the repository"""
    try:
        _ = repo.refs["main"]
        return "main"  # First try 'main'
    except IndexError:
        try:
            _ = repo.refs["master"]
            return "master"  # Then try 'master'
        except IndexError:
            return repo.head.reference.name  # If neither exists, get the branch the HEAD is pointing to


@app.command()
def count_historical(repo_path: Annotated[str, typer.Argument(..., help="Path to the git repository")]):
    """Count total historical lines of Python code in the repository."""
    print(f"Analyzing repository: {repo_path}")
    total_loc: int = count_historical_loc(repo_path)
    print(f"\nTotal historical lines of Python code: {total_loc}")


@app.command()
def analyze_over_time(repo_path: Annotated[str, typer.Argument(..., help="Path to the git repository")]):
    """Analyze a git repository to track Python code size over time with visualization."""
    repo: Repo = Repo(repo_path)
    branch_name: str = get_default_branch(repo)
    print(f"🔍 Using branch: {branch_name}")
    commit_data: "List[Dict[str, Any]]" = []
    print("⏳ Analyzing commits...")
    try:
        commits = list(repo.iter_commits(branch_name))
        from datetime import timezone

        for commit in track(commits, description="Processing commits..."):
            commit_data.append({"hash": commit.hexsha, "dtmExit": datetime.fromtimestamp(commit.committed_date, tz=timezone.utc), "lines": count_python_lines(commit)})
    except Exception as e:
        print(f"❌ Error analyzing commits: {str(e)}")
        return

    import polars as pl
    import plotly.graph_objects as go

    df = pl.DataFrame(commit_data)
    df = df.sort("dtmExit")
    # Create interactive plotly figure with dark theme and all bells and whistles
    fig = go.Figure()
    # Add line chart with gradient fill and sparkle effect
    fig.add_trace(go.Scatter(x=df["dtmExit"], y=df["lines"], mode="lines", line={"width": 3, "color": "#00b4ff"}, fill="tozeroy", fillcolor="rgba(0, 180, 255, 0.2)", name="Lines of Code", hovertemplate="<b>Date:</b> %{x}<br><b>Lines:</b> %{y:,}<extra></extra>"))
    # Add markers for significant points (min, max, last)
    min_idx = df["lines"].arg_min()
    max_idx = df["lines"].arg_max()
    min_point = df.slice(min_idx, 1).to_dicts()[0] if min_idx is not None else {}
    max_point = df.slice(max_idx, 1).to_dicts()[0] if max_idx is not None else {}
    last_point = df.slice(-1, 1).to_dicts()[0]

    # Add markers for significant points
    fig.add_trace(
        go.Scatter(
            x=[min_point["dtmExit"], max_point["dtmExit"], last_point["dtmExit"]],
            y=[min_point["lines"], max_point["lines"], last_point["lines"]],
            mode="markers",
            marker={"size": [10, 14, 12], "color": ["#ff4f4f", "#4fff4f", "#4f4fff"], "line": {"width": 2, "color": "white"}, "symbol": ["circle", "star", "diamond"]},
            name="Key Points",
            hovertemplate="<b>%{text}</b><br>Date: %{x}<br>Lines: %{y:,}<extra></extra>",
            text=[f"🔽 Min: {min_point['lines']:,} lines", f"🔼 Max: {max_point['lines']:,} lines", f"📊 Current: {last_point['lines']:,} lines"],
        )
    )

    # Add annotation only for current point
    # annotations = [
    #     {"x": last_point['date'], "y": last_point['lines'], "text": f"📊 Current: {last_point['lines']:,} lines", "showarrow": True, "arrowhead": 2, "arrowsize": 1,
    #         "arrowwidth": 2, "arrowcolor": "#ffffff", "font": {"size": 14, "color": "#ffffff"}, "bgcolor": "#00b4ff", "bordercolor": "#ffffff",
    #         "borderwidth": 1, "borderpad": 4, "ax": 40, "ay": -40}
    # ]

    # Update layout with dark theme and customizations
    fig.update_layout(
        title={"text": "✨ Python Code Base Size Over Time ✨", "y": 0.95, "x": 0.5, "xanchor": "center", "yanchor": "top", "font": {"size": 24, "color": "white"}},
        xaxis_title="Date 📅",
        yaxis_title="Lines of Code 📝",
        hovermode="closest",
        template="plotly_dark",
        plot_bgcolor="rgba(25, 25, 35, 1)",
        paper_bgcolor="rgba(15, 15, 25, 1)",
        font={"family": "Arial, sans-serif", "size": 14, "color": "white"},  # annotations=annotations,
        autosize=True,
        height=700,
        margin={"l": 80, "r": 80, "t": 100, "b": 80},
        xaxis={"showgrid": True, "gridcolor": "rgba(80, 80, 100, 0.2)", "showline": True, "linecolor": "rgba(200, 200, 255, 0.2)", "tickfont": {"size": 12}},
        yaxis={"showgrid": True, "gridcolor": "rgba(80, 80, 100, 0.2)", "showline": True, "linecolor": "rgba(200, 200, 255, 0.2)", "tickformat": ",", "tickfont": {"size": 12}},
    )

    # Add range slider for date selection
    fig.update_xaxes(rangeslider_visible=True, rangeslider_thickness=0.05)

    # Save as interactive HTML and static image
    plot_dir = Path.home().joinpath("tmp_results", "tmp_images", Path(repo_path).name)
    plot_dir.mkdir(parents=True, exist_ok=True)

    html_path = plot_dir.joinpath("code_size_evolution.html")
    png_path = plot_dir.joinpath("code_size_evolution.png")

    fig.write_html(html_path, include_plotlyjs="cdn")
    fig.write_image(png_path, width=1200, height=700, scale=2)

    print(f"🖼️ Interactive plot saved as {html_path}")
    print(f"🖼️ Static image saved as {png_path}")
    # Print statistics
    print("\n📊 Repository Statistics:")
    print(f"📚 Total commits analyzed: {len(df)}")
    print(f"🔙 Initial line count: {df['lines'][-1]:,}")
    print(f"🔜 Final line count: {df['lines'][0]:,}")
    print(f"📈 Net change: {df['lines'][0] - df['lines'][-1]:,} lines")


def _print_python_files_by_size_impl(repo_path: str) -> "Union[pl.DataFrame, Exception]":
    import polars as pl
    import plotly.graph_objects as go
    import plotly.express as px

    try:
        import os
        if not os.path.exists(repo_path):
            return ValueError(f"Repository path does not exist: {repo_path}")
        # Initialize data storage
        file_data: "List[Dict[str, Union[str, int]]]" = []

        # Walk through the repository
        for root, _, files in os.walk(repo_path):
            # Skip .git directory and other hidden directories
            if ".git" in Path(root).parts or any(part.startswith(".") for part in Path(root).parts):
                continue

            for file in files:
                if file.endswith(".py"):
                    file_path = os.path.join(root, file)
                    try:
                        # Count lines in the file
                        with open(file_path, "r", encoding="utf-8", errors="replace") as f:
                            line_count = sum(1 for _ in f)

                        # Make path relative to repo_path for better display
                        rel_path = os.path.relpath(file_path, repo_path)
                        file_data.append({"filename": rel_path, "lines": line_count})
                    except Exception as e:
                        print(f"⚠️ Warning: Could not read {file_path}: {str(e)}")
                        continue

        # Check if any files were found
        if not file_data:
            return ValueError("❌ No Python files found in the repository")

        # Convert to DataFrame
        df = pl.DataFrame(file_data)

        # Sort DataFrame by line count (descending)
        df = df.sort("lines", descending=True)
        df = df.filter(pl.col("lines") > 0)  # Filter out empty files

        # Add total count
        total_lines = int(df["lines"].sum())
        file_count: int = len(df)

        # Print the DataFrame
        print("\n📊 Python Files Line Count (sorted max to min):")
        print(df)
        print(f"\n📁 Total Python files: {file_count}")
        print(f"📝 Total lines of Python code: {total_lines:,}")

        # Create visualizations with Plotly
        # Only visualize top files (too many files make the chart unreadable)
        top_n: int = min(20, len(df))
        top_files_df = df.head(top_n).clone()

        # Calculate percentage of total for top files
        top_files_df = top_files_df.with_columns((pl.col("lines") / total_lines * 100).round(1).alias("percentage"))

        # Shorten filenames for better display
        import os

        top_files_df = top_files_df.with_columns(pl.col("filename").map_elements(lambda x: os.path.basename(x) if len(x) > 25 else x, return_dtype=pl.Utf8).alias("short_name"))

        # Create bar chart with hover info showing full path
        fig = go.Figure()

        # Add bars with gradient color based on line count
        fig.add_trace(
            go.Bar(
                x=top_files_df["short_name"].to_list(),
                y=top_files_df["lines"].to_list(),
                text=[f"{x}%" for x in top_files_df["percentage"].to_list()],
                textposition="auto",
                hovertemplate="<b>%{customdata}</b><br>Lines: %{y:,}<br>Percentage: %{text}<extra></extra>",
                customdata=top_files_df["filename"],
                marker={"color": top_files_df["lines"], "colorscale": "Viridis", "showscale": True, "colorbar": {"title": "Lines", "thickness": 20, "tickformat": ","}, "line": {"width": 1, "color": "white"}},
                opacity=0.9,
            )
        )

        # Update layout with dark theme
        fig.update_layout(
            title={"text": f"🏆 Top {top_n} Python Files by Size", "y": 0.95, "x": 0.5, "xanchor": "center", "yanchor": "top", "font": {"size": 24, "color": "white"}},
            xaxis_title="File Name 📄",
            yaxis_title="Lines of Code 📝",
            template="plotly_dark",
            plot_bgcolor="rgba(25, 25, 35, 1)",
            paper_bgcolor="rgba(15, 15, 25, 1)",
            font={"family": "Arial, sans-serif", "size": 14, "color": "white"},
            height=700,
            margin={"l": 80, "r": 80, "t": 100, "b": 100},
            xaxis={"tickangle": 45, "showgrid": False, "showline": True, "linecolor": "rgba(200, 200, 255, 0.2)", "tickfont": {"size": 12}},
            yaxis={"showgrid": True, "gridcolor": "rgba(80, 80, 100, 0.2)", "showline": True, "linecolor": "rgba(200, 200, 255, 0.2)", "tickformat": ",", "tickfont": {"size": 12}},
        )

        # Define pie chart figure before conditionally using it
        fig2: "Optional[go.Figure]" = None

        # Add pie chart showing distribution
        if len(df) > top_n:
            # Prepare data for pie chart - top files plus "Others"
            others_lines = df.slice(top_n)["lines"].sum()
            pie_labels = list(top_files_df["short_name"]) + ["Others"]
            pie_values = list(top_files_df["lines"]) + [others_lines]
            pie_customdata = list(top_files_df["filename"]) + [f"Other {len(df) - top_n} files"]

            fig2 = go.Figure()
            fig2.add_trace(go.Pie(labels=pie_labels, values=pie_values, customdata=pie_customdata, textinfo="percent", hovertemplate="<b>%{customdata}</b><br>Lines: %{value:,}<br>Percentage: %{percent}<extra></extra>", marker={"colors": px.colors.sequential.Viridis, "line": {"color": "white", "width": 1}}, hole=0.4, sort=False))

            fig2.update_layout(
                title={"text": "🍩 Python Code Distribution by File", "y": 0.95, "x": 0.5, "xanchor": "center", "yanchor": "top", "font": {"size": 24, "color": "white"}},
                template="plotly_dark",
                plot_bgcolor="rgba(25, 25, 35, 1)",
                paper_bgcolor="rgba(15, 15, 25, 1)",
                font={"family": "Arial, sans-serif", "size": 14, "color": "white"},
                height=700,
                annotations=[{"text": f"Total<br>{total_lines:,}<br>lines", "x": 0.5, "y": 0.5, "font": {"size": 18, "color": "white"}, "showarrow": False}],
            )

        # Save visualizations
        plot_dir = Path.home().joinpath("tmp_results", "tmp_images", Path(repo_path).name)
        plot_dir.mkdir(parents=True, exist_ok=True)

        # Bar chart
        bar_html_path = plot_dir.joinpath("top_files_by_size.html")
        bar_png_path = plot_dir.joinpath("top_files_by_size.png")
        fig.write_html(bar_html_path, include_plotlyjs="cdn")
        fig.write_image(bar_png_path, width=1200, height=700, scale=2)

        print(f"\n🖼️ Interactive bar chart saved as {bar_html_path}")
        print(f"🖼️ Static bar chart saved as {bar_png_path}")

        # Pie chart if available
        if fig2 is not None:
            pie_html_path = plot_dir.joinpath("files_distribution_pie.html")
            pie_png_path = plot_dir.joinpath("files_distribution_pie.png")
            fig2.write_html(pie_html_path, include_plotlyjs="cdn")
            fig2.write_image(pie_png_path, width=1200, height=700, scale=2)

            print(f"🖼️ Interactive pie chart saved as {pie_html_path}")
            print(f"🖼️ Static pie chart saved as {pie_png_path}")

        return df

    except Exception as e:
        return Exception(f"❌ Error analyzing repository: {str(e)}")


@app.command()
def print_python_files_by_size(repo_path: Annotated[str, typer.Argument(..., help="Path to the git repository")]):
    """Print Python files sorted by size with visualizations."""
    result = _print_python_files_by_size_impl(repo_path)
    if isinstance(result, Exception):
        print(f"Error: {result}")
        return


if __name__ == "__main__":
    app()





import platform
from typing import Optional
import tomllib
from pathlib import Path
from machineconfig.utils.accessories import randstr
from machineconfig.utils.path_extended import PathExtended
from machineconfig.utils.options import choose_from_options


def choose_function_or_lines(choice_file: PathExtended, kwargs_dict: dict[str, object]) -> tuple[Optional[str], PathExtended, dict[str, object]]:
    """
    Choose a function to run from a Python file or lines from a shell script.
    
    Returns:
        tuple: (choice_function, choice_file, kwargs_dict)
        - choice_function: The selected function name or None
        - choice_file: The file path (potentially modified for shell scripts)
        - kwargs_dict: Updated kwargs dictionary with user-provided arguments
    """
    choice_function: Optional[str] = None
    
    if choice_file.suffix == ".py":
        from machineconfig.scripts.python.helpers_fire_command.file_wrangler import parse_pyfile
        options, func_args = parse_pyfile(file_path=str(choice_file))
        choice_function_tmp = choose_from_options(msg="Choose a function to run", options=options, fzf=True, multi=False)
        assert isinstance(choice_function_tmp, str), f"choice_function must be a string. Got {type(choice_function_tmp)}"
        choice_index = options.index(choice_function_tmp)
        choice_function = choice_function_tmp.split(" -- ")[0]
        choice_function_args = func_args[choice_index]

        if choice_function == "RUN AS MAIN":
            choice_function = None
        if len(choice_function_args) > 0 and len(kwargs_dict) == 0:
            for item in choice_function_args:
                kwargs_dict[item.name] = input(f"Please enter a value for argument `{item.name}` (type = {item.type}) (default = {item.default}) : ") or item.default
    elif choice_file.suffix == ".sh":
        options = []
        for line in choice_file.read_text(encoding="utf-8").splitlines():
            if line.startswith("#"):
                continue
            if line == "":
                continue
            if line.startswith("echo"):
                continue
            options.append(line)
        chosen_lines = choose_from_options(msg="Choose a line to run", options=options, fzf=True, multi=True)
        choice_file = PathExtended.tmpfile(suffix=".sh")
        choice_file.parent.mkdir(parents=True, exist_ok=True)
        choice_file.write_text("\n".join(chosen_lines), encoding="utf-8")
        choice_function = None
    
    return choice_function, choice_file, kwargs_dict


def get_command_streamlit(choice_file: Path, environment: str, repo_root: Optional[Path]) -> str:
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(("8.8.8.8", 1))
        local_ip_v4 = s.getsockname()[0]
    except Exception:
        local_ip_v4 = socket.gethostbyname(socket.gethostname())
    finally:
        s.close()
    computer_name = platform.node()
    port = 8501
    toml_path: Optional[Path] = None
    toml_path_maybe = choice_file.parent.joinpath(".streamlit/config.toml")
    if toml_path_maybe.exists():
        toml_path = toml_path_maybe
    elif choice_file.parent.name == "pages":
        toml_path_maybe = choice_file.parent.parent.joinpath(".streamlit/config.toml")
        if toml_path_maybe.exists():
            toml_path = toml_path_maybe
    if toml_path is not None:
        print(f"📄 Reading config.toml @ {toml_path}")
        config = tomllib.loads(toml_path.read_text(encoding="utf-8"))
        if "server" in config:
            if "port" in config["server"]:
                port = config["server"]["port"]
        secrets_path = toml_path.with_name("secrets.toml")
        if repo_root is not None:
            secrets_template_path = Path.home().joinpath(f"dotfiles/creds/streamlit/{PathExtended(repo_root).name}/{choice_file.name}/secrets.toml")
            if environment != "" and not secrets_path.exists() and secrets_template_path.exists():
                secrets_template = tomllib.loads(secrets_template_path.read_text(encoding="utf-8"))
                if environment == "ip":
                    host_url = f"http://{local_ip_v4}:{port}/oauth2callback"
                elif environment == "localhost":
                    host_url = f"http://localhost:{port}/oauth2callback"
                elif environment == "hostname":
                    host_url = f"http://{computer_name}:{port}/oauth2callback"
                else:
                    host_url = f"http://{environment}:{port}/oauth2callback"
                try:
                    secrets_template["auth"]["redirect_uri"] = host_url
                    secrets_template["auth"]["cookie_secret"] = randstr(35)
                    secrets_template["auth"]["auth0"]["redirect_uri"] = host_url
                    # save_toml(obj=secrets_template, path=secrets_path)
                except Exception as ex:
                    print(ex)
                    raise ex
    message = f"🚀 Streamlit app is running @:\n1- http://{local_ip_v4}:{port}\n2- http://{computer_name}:{port}\n3- http://localhost:{port}"
    from rich.panel import Panel
    from rich import print as rprint

    rprint(Panel(message))
    exe = f"streamlit run --server.address 0.0.0.0 --server.headless true --server.port {port}"
    # exe = f"cd '{choice_file.parent}'; " + exe
    return exe




import typer
from typing import Annotated


def analyze_repo_development(repo_path: Annotated[str, typer.Argument(..., help="Path to the git repository")]):
    from machineconfig.scripts.python.helpers_repos import count_lines
    from pathlib import Path
    count_lines_path = Path(count_lines.__file__)
    # --project $HOME/code/ machineconfig --group plot
    cmd = f"""uv run --python 3.14 --with "machineconfig[plot]>=7.37" {count_lines_path} analyze-over-time {repo_path}"""
    from machineconfig.utils.code import run_shell_script
    run_shell_script(cmd)


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
"""
Zellij session management utilities for remote operations.
"""

import logging
from typing import Dict, Any, Optional
from pathlib import Path

from rich.console import Console

from machineconfig.cluster.sessions_managers.zellij_utils.remote_executor import RemoteExecutor

logger = logging.getLogger(__name__)
console = Console()


class SessionManager:
    """Handles Zellij session operations on remote machines."""

    def __init__(self, remote_executor: RemoteExecutor, session_name: str, tmp_layout_dir: Path):
        self.remote_executor = remote_executor
        self.session_name = session_name
        self.tmp_layout_dir = tmp_layout_dir

    def copy_layout_to_remote(self, local_layout_file: Path, random_suffix: str) -> str:
        """Copy the layout file to the remote machine and return the remote path."""
        remote_layout_dir = f"~/{self.tmp_layout_dir.relative_to(Path.home())}"
        remote_layout_file = f"{remote_layout_dir}/zellij_layout_{self.session_name}_{random_suffix}.kdl"

        # Create remote directory
        if not self.remote_executor.create_remote_directory(remote_layout_dir):
            raise RuntimeError(f"Failed to create remote directory: {remote_layout_dir}")

        # Copy layout file to remote machine
        copy_result = self.remote_executor.copy_file_to_remote(str(local_layout_file), remote_layout_file)
        if not copy_result["success"]:
            raise RuntimeError(f"Failed to copy layout file to remote: {copy_result['error']}")

        # Enhanced Rich logging
        console.print(f"[bold green]📁 Zellij layout file copied to remote:[/bold green] [yellow]{self.remote_executor.remote_name}[/yellow][cyan]:{remote_layout_file}[/cyan]")
        return remote_layout_file

    def check_zellij_session_status(self) -> Dict[str, Any]:
        """Check if the Zellij session exists and is running."""
        try:
            result = self.remote_executor.run_command("zellij list-sessions", timeout=10)

            if result.returncode == 0:
                sessions = result.stdout.strip().split("\n") if result.stdout.strip() else []
                session_running = any(self.session_name in session for session in sessions)

                return {"zellij_running": True, "session_exists": session_running, "session_name": self.session_name, "all_sessions": sessions, "remote": self.remote_executor.remote_name}
            else:
                return {"zellij_running": False, "error": result.stderr, "session_name": self.session_name, "remote": self.remote_executor.remote_name}

        except Exception as e:
            return {"zellij_running": False, "error": str(e), "session_name": self.session_name, "remote": self.remote_executor.remote_name}

    def start_zellij_session(self, layout_file_path: Optional[str]) -> Dict[str, Any]:
        """Start a Zellij session on the remote machine with the generated layout."""
        try:
            if layout_file_path:
                layout_filename = Path(layout_file_path).name
                remote_layout_file = f"~/{self.tmp_layout_dir.relative_to(Path.home())}/{layout_filename}"
            else:
                raise ValueError("No layout file path provided.")

            # Enhanced Rich logging for session start
            console.print(
                f"[bold cyan]🚀 Starting Zellij session[/bold cyan] [yellow]'{self.session_name}'[/yellow] [dim]on remote[/dim] [bold yellow]'{self.remote_executor.remote_name}'[/bold yellow] [dim]with layout:[/dim] [blue]{remote_layout_file}[/blue]"
            )

            # Start Zellij session with layout
            start_cmd = f"zellij --layout {remote_layout_file} a -b {self.session_name}"
            console.print(f"[dim]Executing:[/dim] [green]{start_cmd}[/green]")
            result = self.remote_executor.run_command(start_cmd, timeout=30)

            if result.returncode == 0:
                console.print(f"[bold green]✅ Zellij session[/bold green] [yellow]'{self.session_name}'[/yellow] [green]started successfully on[/green] [bold yellow]{self.remote_executor.remote_name}[/bold yellow]")
                return {"success": True, "session_name": self.session_name, "remote": self.remote_executor.remote_name, "message": "Session started successfully"}
            else:
                return {"success": False, "error": result.stderr, "session_name": self.session_name, "remote": self.remote_executor.remote_name}

        except Exception as e:
            logger.error(f"Failed to start Zellij session on {self.remote_executor.remote_name}: {e}")
            return {"success": False, "error": str(e), "session_name": self.session_name, "remote": self.remote_executor.remote_name}

    def attach_to_session(self) -> None:
        """Attach to the Zellij session on the remote machine via SSH."""
        self.remote_executor.attach_to_session_interactive(self.session_name)



from machineconfig.utils.source_of_truth import EXCLUDE_DIRS
import fnmatch
from pathlib import Path


def search_files_by_pattern(repo_root: Path, pattern: str) -> list[Path]:
    """Return all files under repo_root whose filename matches the given pattern.

    Notes:
      - Uses glob-style pattern matching (e.g., "*.py", "*test*", "config.*")
      - Skips any paths that reside under directories listed in EXCLUDE_DIRS at any depth.
    """
    matches: list[Path] = []

    def _should_skip_dir(dir_path: Path) -> bool:
        """Check if directory should be skipped based on EXCLUDE_DIRS."""
        return any(part in EXCLUDE_DIRS for part in dir_path.parts)

    def _walk_and_filter(current_path: Path) -> None:
        """Recursively walk directories, filtering out excluded ones early."""
        try:
            if not current_path.is_dir():
                return

            # Skip if current directory is in exclude list
            if _should_skip_dir(current_path):
                return

            for item in current_path.iterdir():
                if item.is_dir():
                    _walk_and_filter(item)
                elif item.is_file() and fnmatch.fnmatch(item.name, pattern):
                    matches.append(item)
        except (OSError, PermissionError):
            # Skip directories we can't read
            return

    _walk_and_filter(repo_root)
    return matches


def search_python_files(repo_root: Path, keyword: str) -> list[Path]:
    """Return all Python files under repo_root whose text contains keyword.

    Notes:
      - Skips any paths that reside under directories listed in EXCLUDE_DIRS at any depth.
      - Errors reading individual files are ignored (decoded with 'ignore').
    """
    keyword_lower = keyword.lower()
    matches: list[Path] = []

    def _should_skip_dir(dir_path: Path) -> bool:
        """Check if directory should be skipped based on EXCLUDE_DIRS."""
        return any(part in EXCLUDE_DIRS for part in dir_path.parts)

    def _walk_and_filter(current_path: Path) -> None:
        """Recursively walk directories, filtering out excluded ones early."""
        try:
            if not current_path.is_dir():
                return

            # Skip if current directory is in exclude list
            if _should_skip_dir(current_path):
                return

            for item in current_path.iterdir():
                if item.is_dir():
                    _walk_and_filter(item)
                elif item.is_file() and item.suffix == ".py":
                    try:
                        if keyword_lower in item.read_text(encoding="utf-8", errors="ignore").lower():
                            matches.append(item)
                    except OSError:
                        # Skip unreadable file
                        continue
        except (OSError, PermissionError):
            # Skip directories we can't read
            return

    _walk_and_filter(repo_root)
    return matches




from types import FunctionType
from typing import Optional, Literal
from machineconfig.utils.schemas.layouts.layout_types import TabConfig, LayoutConfig
from pathlib import Path

def get_fire_tab_using_uv(func: FunctionType, import_module: bool, uv_with: Optional[list[str]], uv_project_dir: Optional[str]) -> tuple[TabConfig, Path]:
    from machineconfig.utils.meta import lambda_to_python_script
    py_script =  lambda_to_python_script(lmb=lambda: func, in_global=True, import_module=import_module)
    from machineconfig.utils.code import get_uv_command_executing_python_script
    command_to_run, py_script_path = get_uv_command_executing_python_script(python_script=py_script, uv_with=uv_with, uv_project_dir=uv_project_dir)
    tab_config: TabConfig = {
        "command": command_to_run,
        "startDir": "$HOME",
        "tabName": func.__name__
    }
    return tab_config, py_script_path
def get_fire_tab_using_fire(func: FunctionType):
    import inspect
    from machineconfig.utils.source_of_truth import CONFIG_ROOT
    import platform
    if platform.system().lower() == "windows":
        wrap_mcfg = CONFIG_ROOT / "scripts/windows/wrap_mcfg.ps1"
        wrap_mcfg = f'& "{wrap_mcfg}"'
    elif platform.system().lower() == "linux" or platform.system().lower() == "darwin":
        wrap_mcfg = CONFIG_ROOT / "scripts/linux/wrap_mcfg"
    else:
        raise ValueError(f"Unsupported platform: {platform.system()}")
    path = Path(inspect.getfile(func))
    path_relative = path.relative_to(Path.home())
    command_to_run = f"""{wrap_mcfg} fire {path_relative} {func.__name__} """
    tab_config: TabConfig = {
        "command": command_to_run,
        "startDir": "$HOME",
        "tabName": func.__name__
    }
    return tab_config



def make_layout_from_functions(functions: list[FunctionType], import_module: bool, tab_configs: list[TabConfig], layout_name: str, method: Literal["script", "fire"]="fire") -> LayoutConfig:
    tabs2artifacts: list[tuple[TabConfig, list[Path]]] = []
    for a_func in functions:
        match method:
            case "script":
                tab_config, artifact_files_1 = get_fire_tab_using_uv(a_func, import_module=import_module, uv_with=None, uv_project_dir=None)
                artifact_files = [artifact_files_1]
            case "fire":
                tab_config = get_fire_tab_using_fire(a_func)
                artifact_files = []
        tabs2artifacts.append((tab_config, artifact_files))
    list_of_tabs = [tab for tab, _ in tabs2artifacts] + tab_configs
    layout_config: LayoutConfig = {
        "layoutName": layout_name,
        "layoutTabs": list_of_tabs,
    }
    return layout_config



# """Trogon
# """

# import click
# from trogon import tui
# from typing import Any


# @tui()
# @click.command()
# @click.option('--description', prompt="Description of the job: ", default=f"Description of running func on remotes", help="Write something that describes what this job is about.")
# @click.option('--update_repo', prompt="Update repo: ", default=False, help="Update the repo on the remote machine.")
# @click.pass_context
# def get_choices(ctx: Any, description: str, update_repo: bool):  # type: ignore
#     return ctx


# if __name__ == '__main__':
#     res = get_choices(standalone_mode=False)  # type: ignore
#     print(res)



#!/usr/bin/env python3
from typing import Optional, Any
from pathlib import Path
import logging
import json
import uuid
from datetime import datetime

from machineconfig.cluster.sessions_managers.wt_utils.remote_executor import WTRemoteExecutor
from machineconfig.cluster.sessions_managers.wt_utils.layout_generator import WTLayoutGenerator
from machineconfig.cluster.sessions_managers.wt_utils.process_monitor import WTProcessMonitor
from machineconfig.cluster.sessions_managers.wt_utils.session_manager import WTSessionManager
from machineconfig.cluster.sessions_managers.wt_utils.status_reporter import WTStatusReporter
from machineconfig.utils.schemas.layouts.layout_types import TabConfig, LayoutConfig

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
TMP_LAYOUT_DIR = Path.home() / "tmp_results" / "wt_layouts"


class WTRemoteLayoutGenerator:
    def __init__(self, layout_config: LayoutConfig, remote_name: str, session_name: str):
        self.remote_name: str = remote_name
        self.session_name: str = session_name
        self.layout_config: LayoutConfig = layout_config.copy()
        self.script_path: Optional[str] = None

        # Initialize modular components
        self.remote_executor = WTRemoteExecutor(remote_name)
        self.layout_generator = WTLayoutGenerator()
        self.process_monitor = WTProcessMonitor(self.remote_executor)
        self.session_manager = WTSessionManager(self.remote_executor, self.session_name, TMP_LAYOUT_DIR)
        self.status_reporter = WTStatusReporter(self.process_monitor, self.session_manager)

    def create_layout_file(self) -> bool:
        """Create Windows Terminal layout file and return success status."""
        tab_count = len(self.layout_config["layoutTabs"])
        logger.info(f"Creating Windows Terminal layout with {tab_count} tabs for remote '{self.remote_name}'")
        
        # Extract tabs from layout_config
        tabs: list[TabConfig] = self.layout_config["layoutTabs"]
        script_content = self.layout_generator.create_wt_script(tabs, self.session_name, window_name=None)
        
        # Write to file
        tmp_layout_dir = Path.home() / "tmp_results" / "wt_layouts" / "remote"
        tmp_layout_dir.mkdir(parents=True, exist_ok=True)
        random_suffix = WTLayoutGenerator.generate_random_suffix(8)
        script_file = tmp_layout_dir / f"wt_layout_{self.session_name}_{random_suffix}.ps1"
        script_file.write_text(script_content, encoding="utf-8")
        self.script_path = str(script_file.absolute())
        
        logger.info(f"✅ Remote layout created: {self.script_path}")
        return True

    # Legacy methods for backward compatibility

    def to_dict(self) -> dict[str, Any]:
        return {"remote_name": self.remote_name, "session_name": self.session_name, "layout_config": self.layout_config, "script_path": self.script_path, "created_at": datetime.now().isoformat(), "class_name": self.__class__.__name__}

    def to_json(self, file_path: Optional[str]) -> str:
        # Generate file path if not provided
        if file_path is None:
            random_id = str(uuid.uuid4())[:8]
            default_dir = Path.home() / "tmp_results" / "wt_sessions" / "serialized"
            default_dir.mkdir(parents=True, exist_ok=True)
            path_obj = default_dir / f"wt_session_{random_id}.json"
        else:
            path_obj = Path(file_path)

        # Ensure .json extension
        if not str(path_obj).endswith(".json"):
            path_obj = path_obj.with_suffix(".json")

        # Ensure parent directory exists
        path_obj.parent.mkdir(parents=True, exist_ok=True)

        # Serialize to JSON
        data = self.to_dict()

        text = json.dumps(data, indent=2, ensure_ascii=False)
        path_obj.write_text(text, encoding="utf-8")

        logger.info(f"✅ Serialized WTRemoteLayoutGenerator to: {path_obj}")
        return str(path_obj)

    @classmethod
    def from_json(cls, file_path: str) -> "WTRemoteLayoutGenerator":
        path_obj = Path(file_path)

        # Ensure .json extension
        if not str(path_obj).endswith(".json"):
            path_obj = path_obj.with_suffix(".json")

        if not path_obj.exists():
            raise FileNotFoundError(f"JSON file not found: {path_obj}")

        # Load JSON data
        text = path_obj.read_text(encoding="utf-8")
        data = json.loads(text)

        # Validate that it's the correct class
        if data.get("class_name") != cls.__name__:
            logger.warning(f"Class name mismatch: expected {cls.__name__}, got {data.get('class_name')}")

        # Create new instance
        instance = cls(
            layout_config=data["layout_config"],
            remote_name=data["remote_name"],
            session_name=data["session_name"]
        )
        instance.script_path = data["script_path"]

        logger.info(f"✅ Loaded WTRemoteLayoutGenerator from: {file_path}")
        return instance

    @staticmethod
    def list_saved_sessions(directory_path: Optional[str]) -> list[str]:
        if directory_path is None:
            dir_path = Path.home() / "tmp_results" / "wt_sessions" / "serialized"
        else:
            dir_path = Path(directory_path)

        if not dir_path.exists():
            return []

        json_files = [f.name for f in dir_path.glob("*.json")]
        return sorted(json_files)


if __name__ == "__main__":
    # Example usage
    sample_tabs: list[TabConfig] = [
        {"tabName": "🤖Bot1", "startDir": "~/code/bytesense/bithence", "command": "python bot1.py --create_new_bot True"},
        {"tabName": "🤖Bot2", "startDir": "~/code/bytesense/bithence", "command": "python bot2.py --create_new_bot True"},
        {"tabName": "📊Monitor", "startDir": "~", "command": "Get-Process | Sort-Object CPU -Descending | Select-Object -First 10"},
        {"tabName": "📝Logs", "startDir": "C:/logs", "command": "Get-Content app.log -Wait"},
    ]

    # Replace 'myserver' with an actual SSH config alias for a Windows machine
    remote_name = "myserver"  # This should be in ~/.ssh/config
    session_name = "test_remote_session"

    try:
        # Create layout config from tabs
        sample_layout: LayoutConfig = {
            "layoutName": "RemoteBots",
            "layoutTabs": sample_tabs
        }
        
        # Create layout using the remote generator
        generator = WTRemoteLayoutGenerator(
            layout_config=sample_layout,
            remote_name=remote_name,
            session_name=session_name
        )
        generator.create_layout_file()
        
        print(f"✅ Remote layout created successfully: {generator.script_path}")

        # Check if Windows Terminal is available on remote
        wt_available = generator.remote_executor.check_wt_available()
        print(f"🖥️  Windows Terminal available on {remote_name}: {'✅' if wt_available else '❌'}")

        # Get remote Windows info
        windows_info = generator.remote_executor.get_remote_windows_info()
        if windows_info.get("wt_available"):
            print(f"📦 Remote system info: {windows_info.get('windows_info', 'Unknown')}")

        # Demonstrate serialization
        print("\n💾 Demonstrating serialization...")
        saved_path = generator.to_json(None)
        print(f"✅ Session saved to: {saved_path}")

        # List all saved sessions
        saved_sessions = WTRemoteLayoutGenerator.list_saved_sessions(None)
        print(f"📋 Available saved sessions: {saved_sessions}")

        # Demonstrate loading (using the full path)
        loaded_generator = WTRemoteLayoutGenerator.from_json(saved_path)
        print(f"✅ Session loaded successfully: {loaded_generator.session_name}")
        print(f"📊 Loaded tabs: {[tab['tabName'] for tab in loaded_generator.layout_config['layoutTabs']]}")

        # Show command preview
        preview = generator.layout_generator.generate_wt_command(sample_tabs)
        print(f"\n📋 Command Preview:\n{preview}")

        # Demonstrate status checking
        print(f"\n🔍 Checking command status on remote '{remote_name}':")
        generator.status_reporter.print_status_report(sample_tabs)

        # Show Windows Terminal overview
        print("\n🖥️  Windows Terminal Overview:")
        generator.status_reporter.print_windows_terminal_overview()

        # Start the session (uncomment to actually start)
        # start_result = generator.session_manager.start_wt_session(generator.script_path)
        # print(f"Session start result: {start_result}")

        # Attach to session (uncomment to attach)
        # generator.session_manager.attach_to_session()

        print("\n▶️  To start this session, run:")
        print("   generator.session_manager.start_wt_session(generator.script_path)")
        print("\n📎 To attach to this session, run:")
        print("   generator.session_manager.attach_to_session()")

    except Exception as e:
        print(f"❌ Error: {e}")
        import traceback

        traceback.print_exc()



# """Run cloud manager.
# """

# from machineconfig.cluster.cloud_manager import CloudManager
# from rich.console import Console  # Add import for Console
# from rich.panel import Panel  # Add import for Panel

# console = Console()

# BOX_WIDTH = 150  # width for box drawing


# def print_section_title(title: str):
#     """Prints a section title formatted nicely with a border and padding."""
#     console = Console()
#     console.print(Panel(title, title_align="left", expand=False))


# def main():
#     console = Console() # Add console initialization
#     console.print(Panel("☁️  Cloud Manager", title_align="left", expand=False))

#     parser.add_argument("-c", "--cloud", help="Rclone Config Name", action="store", type=str, default=None)
#     parser.add_argument("-s", "--serve", help="Start job server", action="store_true", default=False)
#     parser.add_argument("-R", "--reset_local", help="Clear local cache", action="store_true", default=False)
#     parser.add_argument("-r", "--rerun_jobs", help="Re-run jobs by bringing them back from wherever to the queue.", action="store_true", default=False)
#     parser.add_argument("-L", "--release_lock", help="Release lock.", action="store_true", default=False)
#     parser.add_argument("-f", "--queue_failed_jobs", help="Bring failed jobs back to queued jobs for a re-trial.", action="store_true", default=False)
#     parser.add_argument("-m", "--monitor_cloud", help="Monitor workers instead of running a job server.", action="store_true", default=False)
#     parser.add_argument("-j", "--num_jobs", help="Number of jobs the server will run in parallel.", action="store", type=int, default=1)
#     args = parser.parse_args()

#     init_line = f"🔧 Initializing Cloud Manager with {args.num_jobs} worker{'s' if args.num_jobs > 1 else ''}"
#     console.print(Panel(init_line, title_align="left", expand=False))

#     cm = CloudManager(max_jobs=args.num_jobs, cloud=args.cloud, reset_local=args.reset_local)

#     if args.release_lock:
#         line = "🔓 Releasing lock..."
#         console.print(Panel(line, title_align="left", expand=False))
#         cm.claim_lock()
#         cm.release_lock()
#         print("✅ Lock successfully released")

#     if args.queue_failed_jobs:
#         line = "🔄 Requeuing failed jobs..."
#         console.print(Panel(line, title_align="left", expand=False))
#         cm.clean_failed_jobs_mess()
#         print("✅ Failed jobs moved to queue")

#     if args.rerun_jobs:
#         line = "🔁 Rerunning jobs..."
#         console.print(Panel(line, title_align="left", expand=False))
#         cm.rerun_jobs()
#         print("✅ Jobs restarted successfully")

#     if args.monitor_cloud:
#         title = "👁️  STARTING CLOUD MONITOR"
#         console.print(Panel(title, title_align="left", expand=False))
#         cm.run_monitor()

#     if args.serve:
#         title1 = "🚀 STARTING JOB SERVER"
#         run_line = f"💻 Running {args.num_jobs} worker{'s' if args.num_jobs > 1 else ''}"
#         cloud_line = f"☁️  Cloud: {args.cloud if args.cloud else 'Default'}"
#         console.print(Panel(f"{title1}\n{run_line}\n{cloud_line}", title_align="left", expand=False))

#     title = "✅ Cloud Manager finished successfully"
#     console.print(Panel(title, title_align="left", expand=False))
#     import sys
#     sys.exit(0)


# if __name__ == '__main__':
#     main()




from typing import Any, Optional, Callable
from machineconfig.utils.accessories import randstr
from machineconfig.utils.meta import lambda_to_python_script
from pathlib import Path


def print_code(code: str, lexer: str, desc: str, subtitle: str = ""):
    import platform
    try:
        from rich.console import Console
        from rich.panel import Panel
        from rich.syntax import Syntax
        if lexer == "shell":
            if platform.system() == "Windows":
                lexer = "powershell"
            elif platform.system() in ["Linux", "Darwin"]:
                lexer = "sh"
            else:
                raise NotImplementedError(f"Platform {platform.system()} not supported for lexer {lexer}")
        console = Console()
        console.print(Panel(Syntax(code=code, lexer=lexer), title=f"📄 {desc}", subtitle=subtitle), style="bold red")
    except ImportError:
        print(f"--- {desc} ---")
        print(code)
        print(f"--- End of {desc} ---")


def get_uv_command_executing_python_script(python_script: str, uv_with: Optional[list[str]], uv_project_dir: Optional[str]) -> tuple[str, Path]:
    python_file = Path.home().joinpath("tmp_results", "tmp_scripts", "python", randstr() + ".py")
    python_file.parent.mkdir(parents=True, exist_ok=True)
    if uv_with is not None and len(uv_with) > 0:
        uv_with.append("rich")
        uv_with_arg = "--with " + '"' + ",".join(uv_with) + '"'
    else:
        uv_with_arg = "--with rich"
    if uv_project_dir is not None:
        uv_project_dir_arg = "--project" + f' "{uv_project_dir}"'
    else:
        uv_project_dir_arg = ""
    print_code_string = lambda_to_python_script(lambda: print_code(code=python_script, lexer="python", desc="Temporary Python Script", subtitle="Executing via shell script"), in_global=True, import_module=False)
    python_file.write_text(print_code_string + "\n" + python_script, encoding="utf-8")
    shell_script = f"""uv run {uv_with_arg} {uv_project_dir_arg}  {str(python_file)} """
    return shell_script, python_file


def run_lambda_function(lmb: Callable[[], Any], uv_with: Optional[list[str]], uv_project_dir: Optional[str]) -> None:
    code = lambda_to_python_script(lmb, in_global=True, import_module=False)
    uv_command, _py_file = get_uv_command_executing_python_script(python_script=code, uv_with=uv_with, uv_project_dir=uv_project_dir)
    run_shell_script(uv_command)
def run_python_script_in_marimo(py_script: str, uv_project_with: Optional[str]):
    tmp_dir = Path.home().joinpath("tmp_results", "tmp_scripts", "marimo", randstr())
    tmp_dir.mkdir(parents=True, exist_ok=True)
    pyfile = tmp_dir / "marimo_db_explore.py"
    pyfile.write_text(py_script, encoding="utf-8")
    if uv_project_with is not None:
        requirements = f"""--with "marimo" --project {uv_project_with} """
    else:
        requirements = """--with "marimo" """
    fire_line = f"""
cd {tmp_dir}
uv run {requirements} marimo convert {pyfile.name} -o marimo_nb.py
bat marimo_nb.py
uv run  {requirements} marimo edit --host 0.0.0.0 marimo_nb.py
"""
    print_code(code=py_script, desc="Generated Marimo DB Explore Script", lexer="python")
    exit_then_run_shell_script(fire_line)


def run_shell_script(script: str, display_script: bool = True, clean_env: bool = False):
    import tempfile
    import platform
    from rich.console import Console
    from rich.panel import Panel
    from rich.syntax import Syntax

    if platform.system() == "Windows":
        suffix = ".ps1"
        lexer = "powershell"
    else:
        suffix = ".sh"
        lexer = "bash"
    with tempfile.NamedTemporaryFile(mode='w', suffix=suffix, delete=False, encoding='utf-8') as temp_file:
        temp_file.write(script)
        temp_shell_script_path = Path(temp_file.name)
    console = Console()
    if display_script:
        from rich.syntax import Syntax
        console.print(Panel(Syntax(code=script, lexer=lexer), title=f"📄 shell script @ {temp_shell_script_path}", subtitle="shell script being executed"), style="bold red")
    env = {} if clean_env else None
    if platform.system() == "Windows":
        import subprocess
        proc = subprocess.run(f'powershell -ExecutionPolicy Bypass -File "{temp_shell_script_path}"', check=True, shell=True, env=env)
    elif platform.system() == "Linux" or platform.system() == "Darwin":
        import subprocess
        proc = subprocess.run(f"bash {str(temp_shell_script_path)}", check=True, shell=True, env=env)
    else:
        raise NotImplementedError(f"Platform {platform.system()} not supported.")
    # console.print(f"✅  [green]Script executed successfully:[/green] [blue]{temp_script_path}[/blue]")
    if proc.returncode != 0:
        console.print(f"❌  [red]Script execution failed with return code {proc.returncode}:[/red] [blue]{temp_shell_script_path}[/blue]")
    elif proc.returncode == 0:
        console.print(f"✅  [green]Script executed successfully:[/green] [blue]{temp_shell_script_path}[/blue]")
    else:
        console.print(f"⚠️  [yellow]Script executed with warnings (return code {proc.returncode}):[/yellow] [blue]{temp_shell_script_path}[/blue]")
    temp_shell_script_path.unlink(missing_ok=True)
    console.print(f"🗑️  [blue]Temporary script deleted:[/blue] [green]{temp_shell_script_path}[/green]")
    return proc


def exit_then_run_shell_script(script: str, strict: bool = False):
    import os
    from rich.console import Console
    
    console = Console()
    op_program_path = os.environ.get("OP_PROGRAM_PATH", None)
    if op_program_path is not None:
        exists = Path(op_program_path).exists()
    else:
        exists = False
    # three cases: (op_program_path is None, exists=False), (op_program_path is set, exists=False), (op_program_path is set, exists=True)

    if strict:  # we want to assert op_program_path is set and is not an already existing file
        if (op_program_path is None or exists):
            import platform
            if platform.system() == "Windows":
                suffix = ".ps1"
                lexer = "powershell"
            else:
                suffix = ".sh"
                lexer = "bash"
            op_program_path = Path.home().joinpath("tmp_results", "tmp_scripts", "manual_run", f"manual_script_{randstr()}.{suffix}")
            op_program_path.parent.mkdir(parents=True, exist_ok=True)
            op_program_path.write_text(script, encoding="utf-8")
            print_code(script, lexer=lexer, desc="script to run manually")
            console.print("[bold yellow]⚠️  STRICT MODE:[/bold yellow] [cyan]Please run the script manually via your shell by executing:[/cyan]")
            console.print(f"[green]{str(op_program_path)}[/green]")
            console.print("[red]❌ OP_PROGRAM_PATH environment variable is not set in strict mode.[/red]")
            import sys
            sys.exit(1)

    if op_program_path is not None and not exists:
        op_program_path = Path(op_program_path)
        op_program_path.parent.mkdir(parents=True, exist_ok=True)
        op_program_path.write_text(script, encoding="utf-8")
        console.print("[cyan]🚀 Handing over to shell script runner via OP_PROGRAM_PATH:[/cyan]")
        console.print(f"[bold green]{str(op_program_path)}[/bold green]")
        print_code(script, lexer="shell", desc="script to run via OP_PROGRAM_PATH")
    else:
        if op_program_path is not None and exists:
            console.print(f"[yellow]⚠️  OP_PROGRAM_PATH @ {str(op_program_path)} already exists.[/yellow] [cyan]Falling back to direct execution.[/cyan]")
        elif op_program_path is None:
            console.print("[cyan]ℹ️  OP_PROGRAM_PATH is not set.[/cyan] [yellow]Falling back to direct execution.[/yellow]")
        run_shell_script(script)
    import sys
    sys.exit(0)



"""
slidev
"""

from machineconfig.utils.source_of_truth import CONFIG_ROOT
from machineconfig.utils.code import print_code
from machineconfig.utils.path_extended import PathExtended
from machineconfig.utils.terminal import Response
from typing import Annotated, Optional
import typer
import subprocess
import platform

PORT_DEFAULT = 3030

SLIDEV_REPO = PathExtended(CONFIG_ROOT).joinpath(".cache/slidev")
if not SLIDEV_REPO.joinpath("components").exists():
    print("📦 Initializing Slidev repository...")
    subprocess.run(f"cd {SLIDEV_REPO.parent};npm init slidev@latest", check=False, shell=True, text=True)
    print("✅ Slidev repository initialized successfully!\n")


def _execute_with_shell(command: str) -> Response:
    if platform.system() == "Windows":
        completed = subprocess.run(["powershell", "-Command", command], capture_output=True, check=False, text=True)
    else:
        completed = subprocess.run(command, capture_output=True, check=False, text=True, shell=True)
    response = Response.from_completed_process(completed)
    response.print()
    return response


def jupyter_to_markdown(file: PathExtended):
    op_dir = file.parent.joinpath("presentation")
    print("📝 Converting Jupyter notebook to markdown...")

    # https://nbconvert.readthedocs.io/en/latest/nbconvert_library.html
    # from nbconvert.exporters.markdown import MarkdownExporter
    # import nbformat
    # nb = nbformat.read(file, as_version=4)
    # assert isinstance(nb, nbformat.notebooknode.NotebookNode), f"{file} is not a notebook"
    # e = MarkdownExporter(exclude_input=True, exclude_input_prompt=True, exclude_output_prompt=True)
    # body, resources = e.from_notebook_node(nb=nb)
    # op_dir.joinpath("slides_raw.md").write_text(body, encoding="utf-8")
    # for key, value in resources['outputs'].items():

    cmd = f"jupyter nbconvert --to markdown --no-prompt --no-input --output-dir {op_dir} --output slides_raw.md {file}"
    _execute_with_shell(cmd)
    cmd = f"jupyter nbconvert --to html --no-prompt --no-input --output-dir {op_dir} {file}"
    _execute_with_shell(cmd)

    op_file = op_dir.joinpath("slides_raw.md")
    slide_separator = "\n\n---\n\n"
    md = op_file.read_text(encoding="utf-8").replace("\n\n\n\n", slide_separator)
    md = slide_separator.join([item for item in md.split(slide_separator) if bool(item.strip())])
    op_file.with_name("slides.md").write_text(md, encoding="utf-8")
    print(f"✅ Conversion completed! Check the results at: {op_dir}\n")

    return op_dir


def main(
    directory: Annotated[Optional[str], typer.Option("-d", "--directory", help="📁 Directory of the report.")] = None,
    jupyter_file: Annotated[Optional[str], typer.Option("-j", "--jupyter-file", help="📓 Jupyter notebook file to convert to slides. If not provided, slides.md is used.")] = None,
) -> None:
    print("\n" + "=" * 50)
    print("🎥 Welcome to the Slidev Presentation Tool")
    print("=" * 50 + "\n")

    port = PORT_DEFAULT

    if jupyter_file is not None:
        print("📓 Jupyter file provided. Converting to markdown...")
        report_dir = jupyter_to_markdown(PathExtended(jupyter_file))
    else:
        if directory is None:
            report_dir = PathExtended.cwd()
        else:
            report_dir = PathExtended(directory)

    assert report_dir.exists(), f"❌ Directory {report_dir} does not exist."
    assert report_dir.is_dir(), f"❌ {report_dir} is not a directory."

    md_file = report_dir.joinpath("slides.md")
    if not md_file.exists():
        res = report_dir.search("*.md")
        if len(res) == 1:
            md_file = res[0]
        else:
            raise FileNotFoundError(f"❌ slides.md not found in {report_dir}")

    print("📂 Copying files to Slidev repository...")
    for item in report_dir.search():
        item.copy(folder=SLIDEV_REPO, overwrite=True)
    if md_file.name != "slides.md":
        SLIDEV_REPO.joinpath(md_file.name).with_name(name="slides.md", inplace=True, overwrite=True)

    import socket

    try:
        local_ip_v4 = socket.gethostbyname(socket.gethostname() + ".local")
    except Exception:
        print("⚠️ Warning: Could not get local_ip_v4. This might be due to running in a WSL instance.")
        local_ip_v4 = socket.gethostbyname(socket.gethostname())

    print("🌐 Presentation will be served at:")
    print(f"   - http://{platform.node()}:{port}")
    print(f"   - http://localhost:{port}")
    print(f"   - http://{local_ip_v4}:{port}\n")

    program = "npm run dev slides.md -- --remote"
    # PROGRAM_PATH.write_text(program, encoding="utf-8")
    import subprocess

    subprocess.run(program, shell=True, cwd=SLIDEV_REPO)
    print_code(code=program, lexer="bash", desc="Run the following command to start the presentation")


def arg_parser() -> None:
    typer.run(main)


if __name__ == "__main__":
    arg_parser()



"""gh-cli installer"""

import platform
import subprocess
from typing import Optional

from rich.console import Console
from rich.panel import Panel

from machineconfig.utils.installer_utils.installer_class import Installer
from machineconfig.utils.schemas.installer.installer_types import InstallerData

r"""
https://github.com/cli/cli


# as per https://docs.github.com/en/copilot/github-copilot-in-the-cli/using-github-copilot-in-the-cli
# gh auth login
# gh extension install github/gh-copilot

# & 'C:\Program Files\GitHub CLI\gh.exe' extension install github/gh-copilot
# & 'C:\Program Files\GitHub CLI\gh.exe' extension install auth login

"""

config_dict: InstallerData = {
    "appName": "gh",
    "repoURL": "https://github.com/cli/cli",
    "doc": "GitHub CLI",
    "fileNamePattern": {"amd64": {"windows": "gh_{version}_windows_amd64.msi", "linux": "gh_{version}_linux_amd64.deb", "macos": "gh_{version}_macOS_amd64.pkg"}, "arm64": {"windows": "gh_{version}_windows_arm64.msi", "linux": "gh_{version}_linux_arm64.deb", "macos": "gh_{version}_macOS_arm64.pkg"}},
}


console = Console()


def main(version: Optional[str]):
    console.print(
        Panel.fit(
            "\n".join(
                [
                    "[bold magenta]GitHub CLI Installer[/bold magenta]",
                    f"💻 Platform: {platform.system()}",
                    f"🔄 Version: {version or 'latest'}",
                ]
            ),
            title="🔱 GitHub CLI",
            border_style="magenta",
            padding=(1, 2),
        )
    )

    _ = version
    inst = Installer(installer_data=config_dict)
    console.print("[bold cyan]📦 INSTALLATION | Installing GitHub CLI base package...[/bold cyan]")
    inst.install(version=version)

    console.print(
        Panel.fit(
            "🤖 GITHUB COPILOT | Setting up GitHub Copilot CLI extension",
            title="Extension Setup",
            border_style="cyan",
        )
    )

    if platform.system() == "Windows":
        console.print(
            Panel.fit(
                "🪟 WINDOWS SETUP | Configuring GitHub CLI for Windows...",
                border_style="blue",
                title="Platform Setup",
            )
        )
        program = "gh extension install github/gh-copilot"
    elif platform.system() in ["Linux", "Darwin"]:
        system_name = "LINUX" if platform.system() == "Linux" else "MACOS"
        console.print(
            Panel.fit(
                f"🐧 {system_name} SETUP | Configuring GitHub CLI for {platform.system()}...",
                border_style="blue",
                title="Platform Setup",
            )
        )
        program = """
gh extension install github/gh-copilot
"""
    else:
        error_msg = f"Unsupported platform: {platform.system()}"
        console.print(
            Panel.fit(
                f"❌ ERROR | {error_msg}",
                title="Unsupported Platform",
                border_style="red",
            )
        )
        raise NotImplementedError(error_msg)

    program += """
gh auth login --with-token $HOME/dotfiles/creds/git/gh_token.txt
"""
    console.print("[bold]🔐 AUTHENTICATION | Setting up GitHub authentication with token...[/bold]")

    console.print("[bold]🔄 EXECUTING | Running GitHub Copilot extension installation and authentication...[/bold]")
    try:
        subprocess.run(program, shell=True, text=True, check=True)
        console.print("[green]✅ Command executed successfully[/green]")
    except subprocess.CalledProcessError as e:
        console.print(f"❌ [red]Command failed with exit code {e.returncode}[/red]")
        raise

    console.print(
        Panel.fit(
            "\n".join(
                [
                    "✅ SUCCESS | GitHub CLI installation completed",
                    "🚀 GitHub Copilot CLI extension installed",
                    "🔑 Authentication configured with token",
                ]
            ),
            title="Installation Complete",
            border_style="green",
            padding=(1, 2),
        )
    )


if __name__ == "__main__":
    pass



# """Run with sane defaults for a remote machine.
# """

# from machineconfig.cluster.remote_machine import RemoteMachine, RemoteMachineConfig
# from machineconfig.cluster.cloud_manager import CloudManager
# from machineconfig.cluster.loader_runner import WorkloadParams
# from machineconfig.cluster.self_ssh import SelfSSH
# from machineconfig.utils.utils import DEFAULTS_PATH
# from machineconfig.utils.utils2 import read_ini
# from typing import Any, Callable, Union

# _ = WorkloadParams


# try:
#     section = read_ini(DEFAULTS_PATH)['general']
#     to_email_default = section['to_email']
#     email_config_name_default = section['email_config_name']
# except (FileNotFoundError, KeyError, IndexError):
#     to_email_default = 'random@email.com'
#     email_config_name_default = 'enaut'

# try: default_cloud: str = read_ini(DEFAULTS_PATH)['general']['rclone_config_name']
# except (FileNotFoundError, KeyError, IndexError): default_cloud = 'gdrive'


# def run_on_cloud(func: Union[str, Callable[[WorkloadParams], Any]], split: int, reset_cloud: bool = False, reset_local: bool = False):
#     if hasattr(func, '__doc__'): description = str(func.__doc__)
#     else: description = "Description of running an expensive function"
#     config = RemoteMachineConfig(
#         # connection
#         ssh_obj=SelfSSH(),  # overrides ssh_params
#         description=description,  # job_id=, base_dir="",
#         # data
#         copy_repo=False, update_repo=True, install_repo=False, update_essential_repos=True, data=[], transfer_method="cloud", cloud_name=email_config_name_default,
#         # remote machine behaviour
#         open_console=True, notify_upon_completion=True, to_email=to_email_default, email_config_name=email_config_name_default,
#         kill_on_completion=True,
#         launch_method="cloud_manager",
#         # execution behaviour
#         ipython=False, interactive=False, pdb=False, pudb=False, wrap_in_try_except=True,
#         workload_params=None,  # to be added later per sub-job.
#         # resources
#         lock_resources=False, max_simulataneous_jobs=2, parallelize=False, )
#     m = RemoteMachine(func=func, func_kwargs=None, config=config)
#     res = m.submit_to_cloud(split=split, cm=CloudManager(max_jobs=0, reset_local=reset_local), reset_cloud=reset_cloud)
#     return res


# if __name__ == "__main__":
#     pass
